"""
<name>GenExpress</name>
<description>Expression data from GenExpress.</description>
<icon>icons/GenCloud.svg</icon>
<priority>36</priority>
"""

from __future__ import absolute_import

import Orange
import genesis

import sys, os
from collections import defaultdict
import math
from datetime import date
import numpy as np

from Orange.orng import orngEnviron
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

import orangecontrib.bio.widgets.OWPIPAx as OWPIPAx

import requests
from ..utils import compat
from orangecontrib.bio import obiDicty

NAME = "GenExpress"
DESCRIPTION = "Expression data from GenExpress."
ICON = "icons/GenCloud.svg"
PRIORITY = 36

INPUTS = []
OUTPUTS = [("Example table", Orange.data.Table)]

median = obiDicty.median
transformValues = obiDicty.transformValues
averageAttributes = obiDicty.averageAttributes
example_tables = obiDicty.example_tables
CallBack = obiDicty.CallBack

def to_text(x):
    if isinstance(x, dict):
        return str(x["value"])
    else:
        return x

class GenCountConnectionException(Exception):
    pass

class Genesis(object):

    #ignore when joining replicates
    IGNORE_REPLICATE = ["Replicate", "id", "ID", "Name" ]

    #FIXME get this from the server
    LABELS = [
        ("input.alignment.input.reads.var.experiment", "Experiment"),
        ("input.alignment.input.reads.var.sample.time", "Timepoint"),
        ("input.alignment.input.reads.var.replicates.replicate", "Replicate"),
        ("input.alignment.input.reads.var.sample.strain", "Strain"),
        ('input.alignment.input.reads.var.sample.genotype', "Genotype"),
        ("input.alignment.input.reads.var.sample.treatment", "Treatment"),
        ("input.alignment.input.reads.var.sample.growth", "Growth"),
        ("input.alignment.input.genome.static.name", "Genome"),
        ("static.name", "Name"),
        ("unique_id", "ID"),
        ("adapter_type", "Adapter"),
        ("experimenter", "Experimenter"),
        ("band", "Band"),
        ("polya", "Polya"),
        ("primer", "Primer"),
        ("shearing", "Shearing"),
    ]

    def __init__(self, address, cache=None,
                 username='anonymous@genialis.com', password='anonymous', connect=True):

        """
        :param str address: The address of the API.
        :param str username:
        :param str password: Login info; None for public access.
        :param CacheSQLite cache: A cache that stores results locally (an
            :obj:`CacheSQLite`).
        """
        self.address = address
        self.buffer = cache
        self.username = username
        self.password = password
        self._gen = None
        if connect:
            self._gen = genesis.Genesis(username, password, address)
        self._project = None
        self.projectid = None

    @property
    def gen(self):
        if not self._gen:
            try:
                self._gen = genesis.Genesis(self.username, self.password, self.address)
            except:
                raise GenCountConnectionException("Connection needed")
        return self._gen

    @property
    def project(self):
        if not self._project or self._project.id != self.projectid:
            self._project = self.gen.projects()[self.projectid]
        return self._project

    def projects(self, reload=False, bufver="0"):
        def a(self):
            return { k:str(p) for k,p in self.gen.projects().items() }
        return self._buffer_fn("projects" + "|||" + self.username, bufver, reload, a, self)

    def result_types(self, reload=False, bufver="0"):
        """Return a list of available result types."""
        def a(self):
            objects = self.project.data(type__startswith='data:expression:')
            types = {}
            for o in objects:
                an = o.annotation
                for path, a in an.iteritems():
                    if path.startswith('output') and a['type'] == 'basic:file:' \
                        and not path.startswith('output.proc.'):
                            types[a['name']] = a['label']

            # Sort keys by labels
            keys = [typ for typ, label in sorted(types.items(), key=lambda x: x[1])]
            try:
                # Push field exp to the top (exp is considered default)
                keys.remove('exp')
                keys = ['exp'] + keys
            except ValueError:
                # Ignore if no 'exp' field
                pass

            return keys, types
        return self._buffer_fn(self.projectid + "|||" + self.username + "|||" + "result_types", bufver, reload, a, self)

    def results_list(self, rtype, reload=False, bufver="0"):
        """Return a list of available gene expressions for a specific
        result type. Returns a dictionary, where the keys are ID
        and values are dictionaries of sample annotations.

        :param str rtype: Result type to use (see :obj:`result_types`).
        """
        def a(self):
            objects = self.project.data(type__startswith='data:expression:')
            rdict = {}
            for o in objects:
                an = o.annotation
                ok = False
                for path, a in an.iteritems():
                    if path.startswith('output') and a['type'] == 'basic:file:' \
                        and a["name"] == rtype:
                            ok = True
                if ok:
                    rdict[o.id] = an
                    rdict[o.id]["date_modified"] = o.date_modified
            return rdict
        return self._buffer_fn(self.projectid + "|||" + self.username + "|||" + "results_list"  + "|||" + str(rtype), bufver, reload, a, self)

    def _from_buffer(self, addr):
        return self.buffer.get(self.address + "|v1||" + addr)

    def _to_buffer(self, addr, cont, version="0", autocommit=True):
        if self.buffer:
            return self.buffer.add(self.address + "|v1||" + addr, cont, version=version, autocommit=autocommit)

    def _buffer_commit(self):
        if self.buffer:
            self.buffer.commit()

    def _buffer_fn(self, bufkey, bufver, reload, fn, *args, **kwargs):
        """
        If bufkey is already present in buffer, return its contents.
        If not, run function with arguments and save its result
        into the buffer.
        """
        if self._in_buffer(bufkey) == bufver and reload == False:
            #print "IN", bufkey
            res = self._from_buffer(bufkey)
        else:
            #print "NOT IN", bufkey
            res = fn(*args, **kwargs)
            self._to_buffer(bufkey, res, bufver)
        return res

    def _in_buffer(self, addr):
        if self.buffer:
            return self.buffer.contains(self.address + "|v1||" + addr)
        else:
            return False

    def download(self, ids, rtype, reload=False, bufver="0"):
        objdic = self.results_list(rtype)

        downloads = [] #what to download
        for id in ids:
            o = objdic[id]
            field = None
            for path, a in o.iteritems():
                if path.startswith('output') and a['type'] == 'basic:file:' \
                    and not path.startswith('output.proc.'):
                        if a["name"] == rtype:
                            field = a["value"]["file"]
            downloads.append((id, field))

        bufverfn = (lambda x: bufver) if isinstance(bufver, basestring) else bufver

        unbuffered = [] #what is missing
        for id,field in downloads:
            if not self._in_buffer(id + "|||" + rtype) == bufverfn(id) or reload:
                unbuffered.append((id, field))
        unbufferedset = set(unbuffered)

        # newgen = [].__iter__()
        # if unbuffered:
        #     newgen = self.gen.download(unbuffered)
        for id,field in downloads:
            if (id, field) in unbufferedset:
                import gzip
                import StringIO
                response = self.gen.download([id], 'output.' + rtype).next()
                response_gzipped = StringIO.StringIO(response.content)
                response_content = gzip.GzipFile(fileobj=response_gzipped)
                out = []
                for l in response_content.read().split('\n')[1:]:
                    if l:
                        gene, val = l.split('\t')
                        out.append((str(gene), str(val)))
                self._to_buffer(id + "|||" + rtype, out, version=bufverfn(id), autocommit=True)
                yield out
            else:
                yield self._from_buffer(id + "|||" + rtype)

    def get_data(self, ids=None, result_type=None,
                 exclude_constant_labels=False, average=median,
                 callback=None, bufver="0", transform=None,
                 allowed_labels=None, reload=False):
        """
        Return data in a :obj:`Orange.data.Table`. Each feature represents
        a sample and each row is a gene. The feature's ``.attributes``
        contain annotations.

        :param list ids: List of ids as returned by :obj:`results_list`
            if `result_type` is None; list of ids as returned by :obj:`mappings`
            if `result_type` is set.

        :param str result_type: Result template type id as returned by
             :obj:`result_types`.

        :param bool exclude_constant_labels: If a label has the same value
            in whole example table, remove it.

        :param function average: Function that combines multiple reading of
            the same gene on a chip. If None, no averaging is done.
            Function should take a list of floats and return an "averaged"
            float (the default functions returns the median).

        :param function transform: A function that transforms individual values.
            It should take and return a float. Example use: logarithmic
            transformation. Default: None.

        """

        def optcb():
            if callback:
                callback()

        cbc = CallBack(len(ids), optcb, callbacks=10)

        res_list = self.results_list(result_type, reload=reload)

        #annotations
        read = {}
        for a in ids:
            an = res_list[a]
            read[a] = []
            for key, pretty in self.LABELS:
                if key in an:
                    read[a].append((pretty, to_text(an[key])))
        cbc.end()

        download_func = lambda x: self.download(x, result_type, reload=reload,
                                                bufver=bufver)

        cbc = CallBack(len(ids) + 3, optcb,
                       callbacks=99 - 20)
        et = example_tables(ids, spotmap={}, callback=cbc,
                            annots=read,
                            exclude_constant_labels=exclude_constant_labels,
                            chipfn=download_func,
                            allowed_labels=allowed_labels)
        cbc.end()

        cbc = CallBack(2, optcb, callbacks=10)

        #transformation is performed prior to averaging
        if transform != None:
            transformValues(et, fn=transform)  # in place transform
            cbc()

        #if average function is given, use it to join same spotids
        if average != None:
            et = averageAttributes(et, fn=average)
            cbc()

        cbc.end()

        return et


def tfloat(s):
    try:
        return float(s)
    except:
        return None


class MyTreeWidgetItem(QTreeWidgetItem):

    def __init__(self, parent, *args):
        QTreeWidgetItem.__init__(self, parent, *args)
        self.par = parent

    def __contains__(self, text):
        return any(text.upper() in str(self.text(i)).upper() \
                   for i in range(self.columnCount()))

    def __lt__(self, o1):
        col = self.par.sortColumn()
        if col in [8, 9]:  # WARNING: hardcoded column numbers
            return tfloat(self.text(col)) < tfloat(o1.text(col))
        else:
            return QTreeWidgetItem.__lt__(self, o1)


# set buffer file
bufferpath = os.path.join(orngEnviron.directoryNames["bufferDir"], "genesis")

try:
    os.makedirs(bufferpath)
except:
    pass

bufferfile = os.path.join(bufferpath, "database.sq3")

SelectionByKey = OWPIPAx.SelectionByKey
ListItemDelegate = OWPIPAx.ListItemDelegate
SelectionSetsWidget = OWPIPAx.SelectionSetsWidget
SortedListWidget = OWPIPAx.SortedListWidget

# Mapping from PIPAx.results_list annotation keys to Header names.
HEADER = [("_cached", "")] + Genesis.LABELS

# Index of unique_id
ID_INDEX = 10

SORTING_MODEL_LIST = \
    ["Strain", "Experiment", "Genotype",
     "Timepoint", "Growth", "Species",
     "ID", "Name", "Replicate"]


class OWGenExpress(OWWidget):
    settingsList = ["server", "excludeconstant", "username", "password",
                    "joinreplicates", "selectionSetsWidget.selections",
                    "columnsSortingWidget.sortingOrder", "currentSelection",
                    "log2", "experimentsHeaderState", "rtypei", "projecti", "serveri" ]

    def __init__(self, parent=None, signalManager=None, name="Genesis"):
        OWWidget.__init__(self, parent, signalManager, name)
        self.outputs = [("Example table", ExampleTable)]

        self.servers = [
            ('https://dictyexpress.research.bcm.edu/', 'dictyExpress'),
            ('https://cloud.genialis.com/', 'Genialis'),
        ]
        self.username = "anonymous"
        self.password = ""
        self.log2 = False
        self.transpose = False
        self.rtypei = 0
        self.projecti = 0
        self.serveri = 0

        self.selectedExperiments = []
        self.buffer = obiDicty.CacheSQLite(bufferfile)

        self.searchString = ""
        self.excludeconstant = False
        self.joinreplicates = False
        self.currentSelection = None

        self.items = []

        self.experimentsHeaderState = \
                dict(((name, False) for _, name in HEADER[:ID_INDEX + 1]))

        self.result_types = []

        self.controlArea.setMaximumWidth(250)
        self.controlArea.setMinimumWidth(250)

        """
        OWGUI.button(self.controlArea, self, "Reload",
                     callback=self.Reload)
        """
        box = OWGUI.widgetBox(self.controlArea, 'Project')
        self.projectCB = OWGUI.comboBox(box, self, "projecti",
                items=[],
                callback=self.ProjectChosen)

        self.projects = []

        b = OWGUI.widgetBox(self.controlArea, "Selection bookmarks")
        self.selectionSetsWidget = SelectionSetsWidget(self)
        self.selectionSetsWidget.setSizePolicy(QSizePolicy.Preferred,
                                               QSizePolicy.Maximum)
        b.layout().addWidget(self.selectionSetsWidget)

        OWGUI.separator(self.controlArea)

        b = OWGUI.widgetBox(self.controlArea, "Sort output columns")
        self.columnsSortingWidget = SortedListWidget(self)
        self.columnsSortingWidget.setSizePolicy(QSizePolicy.Preferred,
                                                QSizePolicy.Maximum)
        b.layout().addWidget(self.columnsSortingWidget)
        sorting_model = QStringListModel(SORTING_MODEL_LIST)
        self.columnsSortingWidget.setModel(sorting_model)

        self.columnsSortingWidget.sortingOrder = \
                ["Strain", "Experiment", "Genotype", "Timepoint"]

        OWGUI.separator(self.controlArea)

        box = OWGUI.widgetBox(self.controlArea, 'Expression Type')
        self.expressionTypesCB = OWGUI.comboBox(box, self, "rtypei",
                items=[],
                callback=self.UpdateResultsList)

        OWGUI.checkBox(self.controlArea, self, "excludeconstant",
                       "Exclude labels with constant values"
                       )

        OWGUI.checkBox(self.controlArea, self, "joinreplicates",
                       "Average replicates (use median)"
                       )

        OWGUI.checkBox(self.controlArea, self, "log2",
                       "Logarithmic (base 2) transformation"
                       )

        OWGUI.checkBox(self.controlArea, self, "transpose",
                       "Genes as attributes"
                       )

        self.commit_button = OWGUI.button(self.controlArea, self, "&Commit",
                                          callback=self.Commit)
        self.commit_button.setDisabled(True)

        OWGUI.rubber(self.controlArea)

        box = OWGUI.widgetBox(self.controlArea, 'Server')
        OWGUI.comboBox(box, self, "serveri",
                       items=[title for url, title in self.servers],
                       callback=self.ServerChosen)

        OWGUI.lineEdit(box, self, "username", "Username:",
                       labelWidth=100,
                       orientation='horizontal',
                       callback=self.AuthChanged)

        self.passf = OWGUI.lineEdit(box, self, "password", "Password:",
                                    labelWidth=100,
                                    orientation='horizontal',
                                    callback=self.AuthChanged)

        self.passf.setEchoMode(QLineEdit.Password)

        OWGUI.button(self.controlArea, self, "Clear cache",
                     callback=self.clear_cache)

        OWGUI.lineEdit(self.mainArea, self, "searchString", "Search",
                       callbackOnType=True,
                       callback=self.SearchUpdate)

        self.headerLabels = [t[1] for t in HEADER]

        self.experimentsWidget = QTreeWidget()
        self.experimentsWidget.setHeaderLabels(self.headerLabels)
        self.experimentsWidget.setSelectionMode(QTreeWidget.ExtendedSelection)
        self.experimentsWidget.setRootIsDecorated(False)
        self.experimentsWidget.setSortingEnabled(True)

        contextEventFilter = OWGUI.VisibleHeaderSectionContextEventFilter(
                            self.experimentsWidget, self.experimentsWidget
                            )

        self.experimentsWidget.header().installEventFilter(contextEventFilter)
        self.experimentsWidget.setItemDelegateForColumn(0,
                    OWGUI.IndicatorItemDelegate(self, role=Qt.DisplayRole)
                    )

        self.experimentsWidget.setAlternatingRowColors(True)

        self.connect(self.experimentsWidget.selectionModel(),
                 SIGNAL("selectionChanged(QItemSelection, QItemSelection)"),
                 self.onSelectionChanged)

        self.selectionSetsWidget.setSelectionModel(
                            self.experimentsWidget.selectionModel()
                            )

        self.mainArea.layout().addWidget(self.experimentsWidget)

        self.loadSettings()

        self.restoreHeaderState()

        self.connect(self.experimentsWidget.header(),
                     SIGNAL("geometriesChanged()"),
                     self.saveHeaderState)

        self.dbc = None

        self.AuthSet()

        QTimer.singleShot(100, self.ConnectAndUpdate)

        self.resize(800, 600)

    def AuthSet(self):
        if len(self.username):
            self.passf.setDisabled(False)
        else:
            self.passf.setDisabled(True)

    def AuthChanged(self):
        self.AuthSet()
        self.ConnectAndUpdate()

    def ConnectAndUpdate(self):
        self.Connect()
        if self.dbc:
            def get_data_count(project_id):
                # XXX: is there a better way?
                # Note: limit 0 would return all objects
                return self.dbc.gen.api.data.get(case_ids__contains=project_id,
                                                 type__startswith='data:expression:',
                                                 limit=1)['meta']['total_count']

            self.projects = sorted([p for p in self.dbc.projects().items() if
                                    get_data_count(p[0]) > 0], key=lambda x: x[1])
            self.UpdateProjects()
            self.ProjectChosen()
            self.UpdateExperimentTypes()

    def Connect(self):
        self.error(1)
        self.warning(1)

        username = 'anonymous@genialis.com'
        password = 'anonymous'
        url = self.servers[self.serveri][0]

        if self.username:
            username = self.username
            password = self.password

        if username.lower() in ['anonymous@genialis.com', 'anonymous']:
            username = 'anonymous@genialis.com'
            password = 'anonymous'

        self.dbc = None
        self.projects = []
        self.result_types = []

        try:
            self.dbc = Genesis(address=url,
                                  username=username,
                                  password=password,
                                  cache=self.buffer)
        except requests.exceptions.ConnectionError:
            self.dbc = Genesis(address=url,
                                  username=username,
                                  password=password,
                                  connect=False,
                                  cache=self.buffer)
            self.warning(1, "Could not connect to server, working from cache.")
        except Exception, ex:
           self.error(1, "Wrong username or password.")

        self.UpdateProjects()
        self.UpdateExperimentTypes() #clear lists

    def Reload(self):
        self.UpdateExperiments(reload=True)

    def clear_cache(self):
        self.buffer.clear()
        self.Reload()

    def rtype(self):
        """Return selected result template type """
        if self.result_types:
            return self.result_types[self.rtypei]
        else:
            return None

    def UpdateExperimentTypes(self):
        self.expressionTypesCB.clear()
        items = [self.result_types_labels[desc] for desc in self.result_types]
        self.expressionTypesCB.addItems(items)
        #do not update anything if the list is empty
        if len(self.result_types):
            self.rtypei = max(0, min(self.rtypei, len(self.result_types) - 1))

    def UpdateProjects(self):
        self.projectCB.clear()
        items = [desc for pid,desc in self.projects]
        self.projectCB.addItems(items)
        #do not update anathing if the list if empty
        if len(self.projects) > 0:
            self.projecti = max(0, min(self.projecti, len(self.projects) - 1))

    def UpdateExperiments(self, reload=False):

        self.experimentsWidget.clear()

        if not self.dbc or not self.dbc.projectid: #the connection did not succeed
            return

        self.items = []

        self.progressBarInit()

        result_types = []
        result_types_labels = []

        sucind = False  # success indicator for database index

        try:
            result_types, result_types_labels = self.dbc.result_types(reload=reload)
            sucind = True
        except Exception, ex:
            try:
                result_types, result_types_labels = self.dbc.result_types()
                self.warning(0, "Can not access database - using cached data.")
                sucind = True
            except Exception, ex:
                self.error(0, "Can not access database.")

        if sucind:
            self.warning(0)
            self.error(0)

        self.result_types = result_types
        self.result_types_labels = result_types_labels

        self.UpdateExperimentTypes()
        self.UpdateResultsList(reload=reload)

        self.progressBarFinished()

        if self.currentSelection:
            self.currentSelection.select(self.experimentsWidget.selectionModel())

        self.handle_commit_button()

    def ProjectChosen(self, reload=False):
        if self.projects:
            self.dbc.projectid = self.projects[self.projecti][0]
        else:
            self.dbc.projectid = None

        self.UpdateExperiments(reload=reload)

    def ServerChosen(self, reload=False):
        #if self.projects:
        #    self.dbc.projectid = self.projects[self.projecti][0]
        #    self.UpdateExperiments(reload=reload)
        print 'ServerChosen'
        self.ConnectAndUpdate()

    def UpdateResultsList(self, reload=False):

        results_list = {}
        results_list = self.dbc.results_list(self.rtype(), reload=reload)
        try:
            results_list = self.dbc.results_list(self.rtype(), reload=reload)
        except Exception, ex:
            try:
                results_list = self.dbc.results_list(self.rtype())
            except Exception, ex:
                self.error(0, "Can not access database.")

        self.results_list = results_list

        #softly change the view so that the selection stays the same

        items_shown = {}
        for i,item in enumerate(self.items):
            c = str(item.text(ID_INDEX))
            items_shown[c] = i

        items_to_show = set(id_ for id_ in self.results_list)

        add_items = set(items_to_show) - set(items_shown)
        delete_items = set(items_shown) - set(items_to_show)

        i = 0
        while i < self.experimentsWidget.topLevelItemCount():
            it = self.experimentsWidget.topLevelItem(i)
            if str(it.text(ID_INDEX)) in delete_items:
                self.experimentsWidget.takeTopLevelItem(i)
            else:
                i += 1

        delete_ind = set([ items_shown[i] for i in delete_items ])
        self.items = [ it for i, it in enumerate(self.items) if i not in delete_ind ]

        for r_annot in add_items:
            d = defaultdict(lambda: "?", self.results_list[r_annot])
            row_items = [""] + [to_text(d.get(key, "?")) for key, _ in HEADER[1:]]
            row_items[ID_INDEX] = r_annot

            ci = MyTreeWidgetItem(self.experimentsWidget, row_items)
            self.items.append(ci)

        for i in range(len(self.headerLabels)):
            self.experimentsWidget.resizeColumnToContents(i)

        self.wantbufver = lambda x: self.results_list[x]["date_modified"]

        self.UpdateCached()


    def UpdateCached(self):

        if self.wantbufver and self.dbc:

            for item in self.items:
                id = str(item.text(ID_INDEX))
                version = self.dbc._in_buffer(id + "|||" + self.rtype())
                value = " " if version == self.wantbufver(id) else ""
                item.setData(0, Qt.DisplayRole, QVariant(value))

    def SearchUpdate(self, string=""):
        for item in self.items:
            item.setHidden(not all(s in item \
                                   for s in self.searchString.split())
                           )

    def Commit(self):

        pb = OWGUI.ProgressBar(self, iterations=100)

        table = None

        ids = []
        for item in self.experimentsWidget.selectedItems():
            unique_id = str(item.text(ID_INDEX))
            ids.append(unique_id)

        transfn = None
        if self.log2:
            transfn = lambda x: math.log(x + 1.0, 2)

        reverse_header_dict = dict((name, key) for key, name in HEADER)

        hview = self.experimentsWidget.header()
        shownHeaders = [label for i, label in \
                        list(enumerate(self.headerLabels))[1:] \
                        if not hview.isSectionHidden(i)
                        ]

        #allowed_labels = [reverse_header_dict.get(label, label) \
        #for label in shownHeaders]
        allowed_labels = None

        #if self.joinreplicates and "id" not in allowed_labels:
        #    # need 'id' labels in join_replicates for attribute names
        #    allowed_labels.append("id")

        if len(ids):
            table = self.dbc.get_data(ids=ids, result_type=self.rtype(),
                          callback=pb.advance,
                          exclude_constant_labels=self.excludeconstant,
                          bufver=self.wantbufver,
                          transform=transfn,
                          allowed_labels=allowed_labels)

            if self.joinreplicates:
                table = obiDicty.join_replicates(table,
                    ignorenames=self.dbc.IGNORE_REPLICATE,
                    namefn=None,
                    avg=obiDicty.median
                    )

            if self.transpose:
                experiments = [at for at in table.domain.variables]
                attr = [compat.ContinuousVariable(name=ex['DDB'].value) for ex in table]
                metavars = sorted(table.domain.variables[0].attributes.keys())
                metavars = [compat.StringVariable(name=name) for name in metavars]
                domain = compat.create_domain(attr, None, metavars)
                metavars = compat.get_metas(domain)
                X = np.array([[row[exp].value for exp in experiments] for row in table])
                metas = [[exp.attributes[var.name] for var in metavars] for exp in experiments]
                table = compat.create_table(domain, X.transpose().tolist(), None, metas)

            # Sort attributes
            sortOrder = self.columnsSortingWidget.sortingOrder

            all_values = defaultdict(set)
            for at in table.domain.attributes:
                atts = at.attributes
                for name in sortOrder:
                    all_values[name].add(atts.get(reverse_header_dict[name], ""))

            isnum = {}
            for at, vals in all_values.items():
                vals = filter(None, vals)
                try:
                    for a in vals:
                        float(a)
                    isnum[at] = True
                except:
                    isnum[at] = False

            def optfloat(x, at):
                if x == "":
                    return ""
                else:
                    return float(x) if isnum[at] else x

            def sorting_key(attr):
                atts = attr.attributes
                return tuple([optfloat(atts.get(reverse_header_dict[name], ""), name) \
                              for name in sortOrder])

            attributes = sorted(table.domain.attributes,
                                key=sorting_key)

            domain = orange.Domain(attributes, table.domain.classVar)
            domain.addmetas(table.domain.getmetas())
            table = orange.ExampleTable(domain, table)

            from Orange.orng.orngDataCaching import data_hints
            data_hints.set_hint(table, "taxid", "352472")
            data_hints.set_hint(table, "genesinrows", False)

            self.send("Example table", table)

            self.UpdateCached()

        pb.finish()

    def onSelectionChanged(self, selected, deselected):
        self.handle_commit_button()

    def handle_commit_button(self):
        self.currentSelection = \
            SelectionByKey(self.experimentsWidget.selectionModel().selection(),
                           key=(ID_INDEX,))
        self.commit_button.setDisabled(not len(self.currentSelection))

    def saveHeaderState(self):
        hview = self.experimentsWidget.header()
        for i, label in enumerate(self.headerLabels):
            self.experimentsHeaderState[label] = hview.isSectionHidden(i)

    def restoreHeaderState(self):
        hview = self.experimentsWidget.header()
        state = self.experimentsHeaderState
        for i, label in enumerate(self.headerLabels):
            hview.setSectionHidden(i, state.get(label, True))
            self.experimentsWidget.resizeColumnToContents(i)

if __name__ == "__main__":
    app = QApplication(sys.argv)
    obiDicty.verbose = True
    w = OWGenExpress()
    w.show()
    app.exec_()
    w.saveSettings()



"""
Bioinformatics
==============

Bioinformatics widgets for Orange3

"""

NAME = "Bioinformatics"

ICON = "../widgets/icons/Category-Bioinformatics.svg"

BACKGROUND = "light-grass"

intersphinx = (
    ("{DEVELOP_ROOT}/docs/build/html/", None),
    ("http://pythonhosted.org/Orange-Bioinformatics/", None)
)

WIDGET_HELP_PATH = (
    # Used for development.
    # You still need to build help pages using
    # make htmlhelp
    # inside doc folder
    ("{DEVELOP_ROOT}/doc/build/htmlhelp/index.html", None),

    ("http://pythonhosted.org/Orange-Bioinformatics/", "")
)



import sys
import warnings
import io
from collections import defaultdict
from functools import partial
import itertools

import six

from PyQt4.QtGui import (
    QWidget, QComboBox, QListView, QLineEdit, QPlainTextEdit, QRadioButton,
    QButtonGroup, QGroupBox, QCheckBox, QLabel, QFrame, QTabWidget, QSizePolicy,
    QGridLayout, QVBoxLayout, QHBoxLayout, QStackedLayout,
    QStringListModel, QItemSelectionModel,
    QRegExpValidator
)
from PyQt4.QtCore import Qt, QObject, QRegExp, QModelIndex, QThread, QThreadPool, QSize
from PyQt4.QtCore import pyqtSlot as Slot

import Orange

from Orange.widgets.utils import concurrent
from Orange.widgets import widget, gui, settings

from .. import biomart


def is_hidden(tree):
    return (getattr(tree, "hidden", "false") != "false" or
            getattr(tree, "hideDisplay", "false") != "false")


class Control(object):

    """ Base mixin class for query GUI widgets
    """

    def __init__(self, tree, dataset, master):
        """
            :param tree: configuration element
            :type tree: biomart.ConfigurationNode
            :param dataset: dataset
            :type dataset: biomart.BioMartDataset
            :param master: main widget
            :type master: OWBioMart

        """
        self.tree = tree
        self.dataset = dataset
        self.master = master
        self.subControls = []

        if isinstance(self, QObject):
            self.setObjectName(tree.internalName)
            if hasattr(tree, "description"):
                self.setToolTip(tree.description)

    def addSubControl(self, tree, control):
        self.subControls.append((tree, control))

    def registerDelayedCall(self, call):
        self.master.registerDelayedCall(call)

    def pushAction(self, action):
        self.master.pushAction(action)

    def query(self):
        return itertools.chain(*[ctrl.query() for _, ctrl in self.subControls])

    def setControlValue(self, name, value):
        if "." in name:
            name, rest = name.split(".", 1)
            controls = {tree.internalName: control
                        for tree, control in self.subControls}
            ctrl = controls.get("name", None)
            if ctrl:
                ctrl.setControlValue(rest, value)


class UnknownFilter(QWidget, Control):

    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)


class TextFieldFilter(QLineEdit, Control):

    """ A single edit line filter
    """

    def __init__(self, tree, dataset, master, parent=None):
        QLineEdit.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        if hasattr(tree, "regexp"):
            self.setValidator(QRegExpValidator(QRegExp(tree.regexp), self))

        if hasattr(tree, "defaultValue"):
            self.setText(tree.defaultValue)

    def get_filter(self):
        return self.tree.internalName, str(self.text())

    def query(self):
        return [("Filter", self.tree, str(self.text()))]

    def setControlValue(self, name, value):
        self.setText(value)


class IdListFilter(QWidget, Control):

    """ Multiple ids filter
    """

    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        self.tree = tree
        self.dataset = dataset
        self.master = master
        self.setObjectName(tree.internalName)

        self.setLayout(QGridLayout())
        self.textWidget = QPlainTextEdit()  # TODO: Subclass to receive drop events from item model views
        self.layout().addWidget(self.textWidget, 0, 0, 1, 1)

    def value(self):
        """ Return filter value for use in a query
        """
        return str(self.textWidget.toPlainText()).split()

    def get_filter(self):
        return self.tree.internalName, self.value()

    def query(self):
        return [("Filter", self.tree, self.value())]

    def setControlValue(self, name, value):
        if type(value) == list:
            value = "\n".join(value)
        self.textWidget.setPlainText(value)


class RadioBooleanFilter(QWidget, Control):

    """ Boolean filter (Only/Exclude)
    """

    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        self.setLayout(QVBoxLayout())
        self.buttonGroup = QButtonGroup(self)
        self.values = []
        for i, option in enumerate(tree.subelements_top("Option")):
            rb = QRadioButton(option.displayName, self)
            self.buttonGroup.addButton(rb)
            self.buttonGroup.setId(rb, i)
            self.layout().addWidget(rb)
            self.values.append(option.value)
        self.buttonGroup.button(0).setChecked(True)

    def value(self):
        return {"excluded": "%i" % self.buttonGroup.checkedId()}

    def get_filter(self):
        return self.tree.internalName, self.value()

    def query(self):
        return [("Filter", self.tree, self.value())]

    def setControlValue(self, name, value):
        for i, v in enumerate(self.values):
            if v == value:
                button = self.buttonGroup.button(i)
                button.setChecked(True)
                break


class DropDownFilter(QComboBox, Control):

    """ List menu filter
    """

    def __init__(self, tree, dataset, master, parent=None):
        QComboBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        self.options = []
        self.selectedIndex = 0

        if getattr(tree, "graph", "0") == "1":
            self.setOptions(tree.subelements("Option"))
        else:
            self.setOptions(tree.subelements_top("Option"))

        self.currentIndexChanged[int].connect(self.onIndexChange)

    def setOptions(self, options):
        self.options = []
        self.blockSignals(True)
        self.clear()
        for option in options:
            self.addItem(option.displayName)
            self.options.append(option)
        self.selectedIndex = 0
        self.blockSignals(False)
        self.registerDelayedCall(lambda: self.onIndexChange(0))

    def onIndexChange(self, index):
        if self.options:
            option = self.options[index]
            self.selectedIndex = index
            pushActions = option.subelements_top("PushAction")
            for action in pushActions:
                self.master.pushAction(action)

    def value(self):
        option = self.options[self.selectedIndex]
        return option.value

    def query(self):
        return [("Filter", self.tree, self.value())]

    def setControlValue(self, name, value):
        for i, option in enumerate(self.options):
            if option.value == value:
                self.setCurrentIndex(i)


def rows(index_list):
    return map(QModelIndex.row, index_list)


class MultiSelectListFilter(QListView, Control):

    """ Menu list filter with multiple selection
    """

    def __init__(self, tree, dataset, master, parent=None):
        QComboBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        self.setSelectionMode(QListView.ExtendedSelection)
        model = QStringListModel(self)
        self.setModel(model)
        self.setOptions(tree.subelements_top("Option"))

    def setOptions(self, options):
        self.options = []
        for option in options:
            self.options.append(option)
        self.model().setStringList(
            [option.displayName for option in self.options])

    def value(self):
        value = []
        for index in rows(self.selectedIndexes()):
            value.append(self.options[index].value)
        return ",".join(value)

    def query(self):
        return [("Filter", self.tree, self.value())]

    def setControlValue(self, name, value):
        if isinstance(value, str):
            values = value.split(",")
        else:
            values = value
        selection = self.selectionModel()
        for i, option in enumerate(self.options):
            if option.value in values:
                selection.select(self.model().index(i),
                                 QItemSelectionModel.Select)


class DropDownRadioBooleanFilter(QWidget, Control):
    """Container for multiple boolean filters
    """

    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        self.setLayout(QHBoxLayout())
        self.cb = QComboBox(self)

        self.layout().addWidget(self.cb)

        rblayout = QVBoxLayout()
        self.radioButtons = [QRadioButton("Only", self),
                             QRadioButton("Excluded", self)
                             ]

        for b in self.radioButtons:
            rblayout.addWidget(b)

        self.radioButtons[0].setChecked(True)

        self.layout().addLayout(rblayout)

        self.options = []

        self.setOptions(tree.subelements_top("Option"))

    def setOptions(self, options):
        self.cb.clear()
        self.options = []
        for option in options:
            self.cb.addItem(option.displayName)
            self.options.append(option)

        for op, rb in zip(self.options[0].subelements_top("Option"),
                          self.radioButtons):
            rb.setText(op.displayName)
            rb.setChecked(getattr(op, "default", "false") == "true")

    def value(self):
        return {"excluded": "0" if self.radioButtons[0].isChecked() else "1"}

    def query(self):
        filter = self.options[self.cb.currentIndex()]
        filter = biomart.FilterDescription(
            self.tree.registry, "FilterDescription",
            filter.attributes, filter.children)
        return [("Filter", filter, self.value())]

    def setControlValue(self, name, value):
        for i, option in enumerate(self.options):
            if option.internalName == name:
                self.cb.setCurrentIndex(i)
                if value == "Only":
                    self.radioButtons[0].setChecked(True)


class DropDownIdListFilter(QWidget, Control):

    """Container for multiple id list filters
    """

    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        self.setLayout(QVBoxLayout())
        self.setContentsMargins(0, 0, 0, 0)
        self.cb = QComboBox()
        self.idsEdit = QPlainTextEdit()

        self.layout().addWidget(self.cb)
        self.layout().addWidget(self.idsEdit)

        self.options = []
        self.setOptions(tree.subelements_top("Option"))

    def setOptions(self, options):
        self.cb.clear()
        self.options = []
        for option in options:
            self.cb.addItem(option.displayName)
            self.options.append(option)

    def value(self):
        return str(self.idsEdit.toPlainText()).split()

    def query(self):
        filter = self.options[self.cb.currentIndex()]
        filter = biomart.FilterDescription(
            self.tree.registry, "FilterDescription",
            filter.attributes, filter.children)
        return [("Filter", filter, self.value())]

    def setControlValue(self, name, value):
        if isinstance(value, list):
            value = "\n".join(value)

        for i, op in enumerate(self.options):
            if name == op.internalName:
                self.cb.setCurrentIndex(i)
                self.idsEdit.setPlainText(value)


class CollectionWidget(QGroupBox, Control):
    NO_FLAGS = 0
    FILTER_FLAG = 1
    ATTRIBUTE_FLAG = 2
    SINGLE_FILTER_FLAG = 4

    def __init__(self, tree, dataset, master, parent=None):
        QGroupBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        self.setFlat(True)
        self.attributes = []
        self.filters = []

    def setCollection(self, tree, dataset, flags=0):
        self.tree = tree
        self.dataset = dataset
        self.collectionFlags = flags


def isFilterCollection(tree):
    """ In case all AttributeDescription nodes contain pointers to filters
    (case of downstream/upstream_flank)
    """
    if tree.tag == "FilterCollection":
        return True
    return all(["pointerFilter" in desc.attributes
                for desc in tree.elements("AttributeDescription")])


class AttributeWidget(QCheckBox, Control):

    """ Single attribute selection widget
    """

    def __init__(self, tree, dataset, master, parent=None):
        QCheckBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)

        self.setText(getattr(tree, "displayName", ""))
        self.setChecked(getattr(tree, "default", "false") == "true")

        if hasattr(tree, "description"):
            self.setToolTip(tree.description)

    def query(self):
        if self.isChecked():
            return [("Attribute", self.tree, self.isChecked())]
        else:
            return []

    def setControlValue(self, name, value):
        self.setChecked(value == "true")


class AttributeCollectionWidget(CollectionWidget):

    def __init__(self, tree, dataset, master, parent=None):
        CollectionWidget.__init__(self, tree, dataset, master, parent)
        self.setLayout(QGridLayout(self))
        self.setCollection(tree, dataset)

    def setCollection(self, collection, dataset, flags=0):
        CollectionWidget.setCollection(self, collection, dataset, flags)
        self.setTitle(getattr(collection, "displayName", ""))

        if hasattr(collection, "description"):
            self.setToolTip(collection.description)

        attributes = [attr for attr in collection.elements("AttributeDescription")
                      if not is_hidden(attr)]

        numAttrs = max(len(attributes) / 2 + 1, 1)
        i = 0
        for attr in attributes:
            if attr.is_pointer():
                try:
                    dataset, newattr = attr.get_pointed()
                except ValueError as ex:
                    newattr = None

                if not newattr:
                    continue
                attr = newattr

            attr_widget = AttributeWidget(attr, self.dataset, self.master, self)
            self.layout().addWidget(attr_widget, i % numAttrs, i / numAttrs)
            self.addSubControl(attr, attr_widget)
            i += 1


def filterType(filter):
    return (getattr(filter, "displayType", ""),
            getattr(filter, "style", ""),
            getattr(filter, "multipleValues", ""))


class FilterCollectionWidget(CollectionWidget):

    def __init__(self, tree, dataset, master, parent=None):
        CollectionWidget.__init__(self, tree, dataset, master, parent)
        self.setLayout(QGridLayout())

        self.layout().setContentsMargins(5, 5, 5, 5)
        self.layout().setColumnStretch(0, 10)
        self.layout().setColumnStretch(1, 10)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        self.setCollection(tree, dataset)

    def setCollection(self, collection, dataset, flags=0):
        CollectionWidget.setCollection(self, collection, dataset, flags)
        filters = [f for f in collection.elements("FilterDescription") if not is_hidden(f)] + \
                  [f for f in collection.elements("AttributeDescription") if not is_hidden(f)]  # in case of pointers to filters (upstream/downstream flank)
        self.enableCB = QCheckBox(getattr(collection, "displayName", ""), self)

        if hasattr(collection, "description"):
            self.setToolTip(collection.description)

        if len(filters) == 1 and (not hasattr(filters[0], "displayName") or \
                                  getattr(filters[0], "displayName", "") == getattr(collection, "displayName", "")):
            flags = flags | self.SINGLE_FILTER_FLAG

        i = 0
        if not flags & self.SINGLE_FILTER_FLAG:
            self.layout().addWidget(self.enableCB, 0, 0)
            i += 1

        for filter in filters:
            fType = getattr(filter, "type", None)
            if filter.is_pointer():
                try:
                    dataset, newfilter = filter.get_pointed()
                except ValueError as ex:
                    newfilter = None

                if not newfilter:
                    continue
                filter = newfilter
                fType = getattr(filter, "type", None) if fType is None else fType
            label, filter_widget = self.buildFilter(filter, flags, fTypeHint=fType)
            if isinstance(label, six.string_types):
                label = QLabel(label)

            self.layout().addWidget(label, i, 0)
            self.layout().addWidget(filter_widget, i, 1)
            i += 1

            self.addSubControl(filter, filter_widget)
        if self.layout().count() == 0:
            self.layout().addWidget(self.enableCB, 0, 0)

    def buildFilter(self, filter, flags, fTypeHint=None):
        if flags & self.SINGLE_FILTER_FLAG:
            label = self.enableCB
        else:
            label = getattr(filter, "displayName", "")

        fType = filterType(filter)
        if fTypeHint == "drop_down_basic_filter":
            fType = ("list", "menu", "")
        filter_widget = None

        if fType == ("text", "", ""):
            filter_widget = TextFieldFilter(filter, self.dataset, self.master, self)

        elif fType == ("text", "", "1"):
            filter_widget = IdListFilter(filter, self.dataset, self.master, self)

        elif fType == ("list", "radio", ""):
            filter_widget = RadioBooleanFilter(filter, self.dataset, self.master, self)

        elif fType == ("list", "menu", ""):
            filter_widget = DropDownFilter(filter, self.dataset, self.master, self)

        elif fType == ("list", "menu", "1"):
            filter_widget = MultiSelectListFilter(filter, self.dataset, self.master, self)

        elif fType == ("container", "", ""):
            fType = set(map(filterType, filter.elements_top("Option")))
            if len(fType) != 1:
                warnings.warn("Multiple filter types in a single container!" + str(fType))
            fType = fType.pop()
            if fType[0] == "text":  # ("text", "", "1"):
                filter_widget = DropDownIdListFilter(filter, self.dataset, self.master, self)
            elif fType == ("list", "radio", ""):
                filter_widget = DropDownRadioBooleanFilter(filter, self.dataset, self.master, self)

        if filter_widget is None:
            warnings.warn("Unknown filter type '%s' %s'" % (repr(fType), repr(filter)))
            filter_widget = UnknownFilter(filter, self.dataset, self.master, self)

        filter_widget.setMaximumWidth(400)
        return label, filter_widget

    def query(self):
        if self.enableCB.isChecked():
            return CollectionWidget.query(self)
        else:
            return []


class GroupWidget(QGroupBox, Control):

    def __init__(self, tree, dataset, master, parent=None):
        QGroupBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        self.setLayout(QVBoxLayout())

        if tree:
            self.setGroup(tree)

    def setGroup(self, group):
        self.setTitle(getattr(group, "displayName", ""))
        if hasattr(group, "description"):
            self.setToolTip(group.description)
        self.setCheckable(True)
        self.setChecked(True)

        if group.tag == "FilterGroup":
            collections = group.elements("FilterCollection")
        else:
            collections = group.elements("AttributeCollection")
        for collection in collections:
            if isFilterCollection(collection):
                collection_widget = FilterCollectionWidget(collection, self.dataset, self.master, self)
            else:
                collection_widget = AttributeCollectionWidget(collection, self.dataset, self.master, self)

            self.layout().addWidget(collection_widget)
            self.addSubControl(collection, collection_widget)


class PageWidget(QFrame, Control):

    def __init__(self, tree, dataset, master, parent=None):
        QFrame.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        self.setLayout(QVBoxLayout())
        self.outFormats = getattr(tree, "outFormats", "tsv")

        if tree:
            self.setPage(tree)

    def setPage(self, tree):
        if tree.tag == "FilterPage":
            groups = tree.elements("FilterGroup")
        else:
            groups = tree.elements("AttributeGroup")

        for group in groups:
            group_widget = GroupWidget(group, self.dataset, self.master, self)
            self.layout().addWidget(group_widget)
            self.addSubControl(group, group_widget)

        self.layout().addStretch(10)

        if hasattr(tree, "description"):
            self.setToolTip(tree.description)


class OWBioMart(widget.OWWidget):
    name = "BioMart"
    description = "Query BioMart service"
    icon = "../widgets/icons/BioMart.svg"
    priority = 2010

    outputs = [("Data", Orange.data.Table)]

    SHOW_FILTERS = True

    selectedDataset = settings.Setting(0)

    def __init__(self, parent=None):
        super().__init__(parent)

        self.selectedDatabase = 0
        self.uniqueRows = True

        gui.button(gui.widgetBox(self.controlArea, "Cache", addSpace=True),
                   self, "Clear cache",
                   tooltip="Clear saved query results",
                   callback=self.clearCache)

        self.martsCombo = gui.comboBox(
            self.controlArea, self, "selectedDatabase", "Database",
            callback=self.setSelectedMart,
            addSpace=True)
        self.martsCombo.setMaximumWidth(250)

        self.datasetsCombo = gui.comboBox(
            self.controlArea, self, "selectedDataset", "Dataset",
            callback=self.setSelectedDataset,
            addSpace=True)

        self.datasetsCombo.setMaximumWidth(250)

        gui.rubber(self.controlArea)

        box = gui.widgetBox(self.controlArea, "Results")
        gui.checkBox(
            box, self, "uniqueRows", "Unique results only",
            tooltip="Return unique results only.",)

        self.commitButton = gui.button(
            box, self, "Get Results", callback=self.commit,
            tooltip="Query the BioMart server and output the results",
            autoDefault=True)

        self.commitButton.setEnabled(False)

        self.mainWidget = gui.widgetBox(
            self.mainArea, orientation=QStackedLayout())

        self.mainTab = QTabWidget()

        self.mainWidget.layout().addWidget(self.mainTab)

        self.attributesConfigurationBox = gui.createTabPage(self.mainTab, "Attributes")

        if self.SHOW_FILTERS:  # ??
            self.filtersConfigurationBox = gui.createTabPage(self.mainTab, "Filters")

        self.error(0)
        self.setEnabled(False)
        self._executor = concurrent.ThreadExecutor(
            threadPool=QThreadPool(maxThreadCount=2)
        )
        self._task = task = concurrent.Task(function=self._get_registry)
        task.resultReady.connect(self.setBioMartRegistry)
        task.exceptionReady.connect(self._handleException)
        self._executor.submit(task)

        self._afterInitQueue = []

        try:
            from Bio import SeqIO
            self.hasBiopython = True
        except ImportError:
            self.warning(100, "Biopython package not found.\nTo retrieve FASTA sequence data from BioMart install Biopython.")
            self.hasBiopython = False

    def sizeHint(self):
        return QSize(800, 600)

    @staticmethod
    def _get_registry(url=None, precache=True):
        con = biomart.BioMartConnection(url, timeout=30)
        reg = biomart.BioMartRegistry(con)
        if precache:
            _ = reg.marts()
        return reg

    @Slot(Exception)
    def _handleException(self, exception):
        assert(QThread.currentThread() is self.thread())
        print("Task failed with:", exception, file=sys.stderr)
        self.error(0, str(exception))
        self.setEnabled(True)

    @Slot(object)
    def setBioMartRegistry(self, registry):
        assert(QThread.currentThread() is self.thread())
        self.setEnabled(True)
        self.registry = registry
        self.marts = [mart for mart in self.registry.marts()
                      if getattr(mart, "visible", "0") != "0"]

        for mart in self.marts:
            self.martsCombo.addItem(mart.displayName)

    def setSelectedMart(self):
        self.mart = self.marts[self.selectedDatabase]

        self.error(0)
        self.setEnabled(False)

        self._task = task = concurrent.Task(function=self.mart.datasets)
        task.resultReady.connect(self.setBioMartDatasets)
        task.exceptionReady.connect(self._handleException)
        self._executor.submit(task)

    @Slot(object)
    def setBioMartDatasets(self, datasets):
        assert(QThread.currentThread() is self.thread())
        self.setEnabled(True)
        self.datasets = [data for data in datasets if
                         getattr(data, "visible", "0") != "0"]
        self.datasetsCombo.clear()
        self.datasetsCombo.addItems([data.displayName for data in self.datasets])

    def setSelectedDataset(self):
        self.dataset = self.datasets[self.selectedDataset]
        self.error(0)
        self.setEnabled(False)

        def get_configuration(dataset):
            connection = dataset.connection
            stream = connection.configuration(
                dataset=dataset.internalName,
                virtualSchema=dataset.virtualSchema)
            response = stream.read()
            return response

        self._task = task = concurrent.Task(
            function=partial(get_configuration, self.dataset))

        task.resultReady.connect(self.setBioMartConfiguration)
        task.exceptionReady.connect(self._handleException)

        self._executor.submit(task)

    @Slot(object)
    def setBioMartConfiguration(self, configuration):
        assert(QThread.currentThread() is self.thread())
        self.setEnabled(True)
        # parse the xml in the main thread (a long time ago this step was
        # done in a thread but would frequently cause `expat` to segfault.
        doc = biomart.parseXML(io.BytesIO(configuration))
        config = list(doc.elements("DatasetConfig"))[0]
        configuration = biomart.DatasetConfig(
            self.registry, config.tag, config.attributes, config.children)

        self.clearConfiguration()

        self.configuration = configuration

        def hidden(tree):
            return getattr(tree, "hidden", "false") != "false" or \
                   getattr(tree, "hideDisplay", "false") != "false"

        self.attributePagesTabWidget = tabs = gui.tabWidget(self.attributesConfigurationBox)

        for page in configuration.elements("AttributePage"):
            if not hidden(page):
                page_widget = PageWidget(page, self.dataset, self)
                gui.createTabPage(tabs, getattr(page, "displayName", ""),
                                  widgetToAdd=page_widget, canScroll=True)

        if self.SHOW_FILTERS:
            self.filterPagesTabWidget = tabs = gui.tabWidget(self.filtersConfigurationBox)
            for page in configuration.elements("FilterPage"):
                if not hidden(page):
                    page_widget = PageWidget(page, self.dataset, self)
                    gui.createTabPage(tabs, getattr(page, "displayName", ""),
                                      widgetToAdd=page_widget, canScroll=True)

        self.afterInit()

        self.commitButton.setEnabled(True)

    def clearConfiguration(self):
        self.mainTab.deleteLater()

        self.mainTab = QTabWidget()
        self.mainWidget.layout().addWidget(self.mainTab)
        self.mainWidget.layout().setCurrentWidget(self.mainTab)

        self.attributesConfigurationBox = gui.createTabPage(self.mainTab, "Attributes")
        if self.SHOW_FILTERS:
            self.filtersConfigurationBox = gui.createTabPage(self.mainTab, "Filters")

    def commit(self):
        pageconf = self.attributePagesTabWidget.currentWidget().widget()
        format = pageconf.outFormats

        self.error(100)
        if not self.hasBiopython and format.lower() == "fasta":
            self.error(100, "Cannot parse FASTA format")
            return

        query = pageconf.query()
        bydatasets = defaultdict(lambda: ([], []))

        for conftype, tree, val in query:
            dataset = self.dataset

            if conftype == "Attribute":
                bydatasets[dataset][0].append(tree.internalName)
            elif conftype == "Filter":
                bydatasets[dataset][1].append((tree.internalName, val))

        if self.SHOW_FILTERS:
            pageconf = self.filterPagesTabWidget.currentWidget().widget()
            query = pageconf.query()

            for conftype, tree, val in query:
                dataset = self.dataset

                if conftype == "Attribute":
                    bydatasets[dataset][0].append(tree.internalName)
                elif conftype == "Filter":
                    bydatasets[dataset][1].append((tree.internalName, val))

        query = self.registry.query(
            format="TSV" if "tsv" in format.lower() else format.upper(),
            uniqueRows=self.uniqueRows)

        for dataset, (attributes, filters) in bydatasets.items():
            query.set_dataset(dataset if dataset else self.dataset)
            for attr in attributes:
                query.add_attribute(attr)
            for filter, value in filters:
                query.add_filter(filter, value)

        self.error(0)
        self.setEnabled(False)
        self._task = task = concurrent.Task(function=query.get_table)
        task.resultReady.connect(self.dataReady)
        task.exceptionReady.connect(self._handleException)
        self._executor.submit(task)

    def dataReady(self, data):
        self.setEnabled(True)
        self.send("Data", data)

    def pushAction(self, action):
        ref = action.ref
        ref_widget = self.findChild(QWidget, ref)
        if hasattr(ref_widget, "setOptions"):
            ref_widget.setOptions(action.subelements_top("Option"))

    def registerDelayedCall(self, call):
        self._afterInitQueue.append(call)

    def afterInit(self):
        while self._afterInitQueue:
            call = self._afterInitQueue.pop(0)
            call()

    def clearCache(self):
        self.registry.connection.clear_cache()


def test_main(argv=sys.argv):
    from PyQt4.QtGui import QApplication
    app = QApplication(sys.argv)
    ow = OWBioMart()
    ow.show()
    ow.raise_()

    data = Orange.data.Table("brown-selected")
    ow.setData(data)
    r = app.exec_()
    ow.saveSettings()
    return r

if __name__ == "__main__":
    sys.exit(test_main())



# compatibility with Orange 2 and 3

import Orange

OR3 = True if Orange.__version__ >= "3" else False

if OR3:
    from Orange.data import DiscreteVariable, ContinuousVariable, StringVariable
else:
    from Orange.feature import Discrete as DiscreteVariable
    from Orange.feature import Continuous as ContinuousVariable
    from Orange.feature import String as StringVariable

if OR3:
    import numpy
    unknown = NAN = float("nan")
    isunknown = numpy.isnan
else:
    unknown = NAN = '?'
    isunknown = lambda v: v == NAN


def create_domain(at, cl, metas):
    if OR3:
        return Orange.data.Domain(at, cl, metas=metas)
    else:
        domain  = Orange.data.Domain(at, cl)
        if metas:
            if isinstance(metas, dict):
                metas = sorted(metas.items())
            else:
                metas = zip([ StringVariable.new_meta_id() for _ in metas ], metas)
            domain.add_metas(dict((StringVariable.new_meta_id(), ma) for mi, ma in metas))
        return domain


def create_table(domain, X, Y, metas):
    classvar = domain.class_var
    if OR3:
        metaatts = domain.metas
        if Y:
            Y = numpy.array([[classvar.to_val(row)] for row in Y],
                            dtype=float)
        if metas:
            metas = numpy.array([[c.to_val(v) for c, v in zip(metaatts, row)]
                                 for row in metas],
                                dtype=object)
        data = Orange.data.Table(domain, numpy.asarray(X), Y=Y, metas=metas)
    else:
        metaatts = [ at for i,at in sorted(domain.getmetas().items()) ]
        insts = []
        for i in range(len(X)):
            if Y:
                inst = Orange.data.Instance(domain, X[i] + [ Y[i] ])
            else:
                inst = Orange.data.Instance(domain, X[i])
            if metaatts:
                for ma,mv in zip(metaatts, metas[i]):
                    inst[ma] = mv
            insts.append(inst)
        data = Orange.data.Table(domain, insts)
    return data

def get_metas(domain):
    if OR3:
        return domain.metas
    else:
        return [at for i, at in sorted(domain.getmetas().items())]



from __future__ import absolute_import, with_statement

try:
    import cPickle as pickle
except:
    import pickle
    
import os, tempfile, sys
from collections import defaultdict
import datetime

from ..utils import serverfiles

import six

try:
    from Orange.utils import environ
except ImportError:
    from ..utils import environ

try:
    basestring
except:
    basestring = str

from .. import go as obiGO, kegg as obiKEGG, taxonomy as obiTaxonomy
from .. import dicty 
obiDictyMutants = dicty.phenotypes
from .. import omim as obiOMIM
from .. import go as obiGO

try:
    from . import transform
except:
    pass  #not yet available in Orange3

sfdomain = "gene_sets"

def nth(l,n):
    return [ a[n] for a in l]

class NoGenesetsException(Exception): pass

def goGeneSets(org):
    """Returns gene sets from GO."""
    ontology = obiGO.Ontology()
    annotations = obiGO.Annotations(org, ontology=ontology)

    genesets = []
    link_fmt = "http://amigo.geneontology.org/cgi-bin/amigo/term-details.cgi?term=%s"
    for termn, term in ontology.terms.items():
        genes = annotations.get_all_genes(termn)
        hier = ("GO", term.namespace)
        if len(genes) > 0:
            gs = GeneSet(id=termn, name=term.name, genes=genes, hierarchy=hier, organism=org, link=link_fmt % termn)
            genesets.append(gs)

    return GeneSets(genesets)

def strornone(x):
    return str(x) if x != None else x

def keggGeneSets(org):
    """
    Returns gene sets from KEGG pathways.
    """

    kegg = obiKEGG.KEGGOrganism(org)

    genesets = []
    for id in kegg.pathways():
        pway = obiKEGG.KEGGPathway(id)
        hier = ("KEGG","pathways")
        if pway.pathway_attributes():
            gs = GeneSet(id=id,
                                 name=pway.title,
                                 genes=kegg.get_genes_by_pathway(id),
                                 hierarchy=hier,
                                 organism=org,
                                 link=pway.link)
            genesets.append(gs)

    return GeneSets(genesets)

def dictyMutantSets():
    """
    Return dicty mutant phenotype gene sets from Dictybase
    """
    link_fmt = "http://dictybase.org/db/cgi-bin/dictyBase/SC/scsearch.pl?searchdb=strains&search_term=%s&column=all&B1=Submit" 
    #genesets = [GeneSet(id=mutant.name, name=mutant.descriptor, genes=obiDictyMutants.mutant_genes(mutant), hierarchy=("Dictybase", "Mutants"), organism="352472", # 352472 gathered from obiGO.py code_map -> Dicty identifier
    #                    link=(link_fmt % mutant.name if mutant.name else None)) \
    #                    for mutant in obiDictyMutants.mutants()]
 
    genesets = [GeneSet(id=phenotype, name=phenotype, genes=[obiDictyMutants.mutant_genes(mutant)[0] for mutant in mutants], hierarchy=("Dictybase", "Phenotypes"), organism="352472", # 352472 gathered from obiGO.py code_map -> Dicty identifier
                        link="") \
                        for phenotype, mutants in obiDictyMutants.phenotype_mutants().items()]

    return GeneSets(genesets)

def cytobandGeneSets():
    """
    Create cytoband gene sets from Stanford Microarray Database
    """
    import urllib2

    url = "http://www-stat.stanford.edu/~tibs/GSA/cytobands-stanford.gmt"
    stream = urllib2.urlopen(url)
    data = stream.read().splitlines()

    genesets = []
    for band in data:
        b = band.split("\t")
        genesets.append(GeneSet(id=b[0], name=b[1], genes=b[2:] if b[2:] else [], hierarchy=("Cytobands",), organism="9606", link=""))          

    return GeneSets(genesets)

def reactomePathwaysGeneSets():
    """
    Prepare human pathways gene sets from reactome pathways
    """
    import urllib
    import io
    from zipfile import ZipFile

    url = urllib.urlopen("http://www.reactome.org/download/current/ReactomePathways.gmt.zip")
    memfile = io.BytesIO(url.read())
    with ZipFile(memfile, "r") as myzip:
        f = myzip.open("ReactomePathways.gmt")
        content = f.read().splitlines()      

    genesets = [GeneSet(id=path.split("\t")[0], name=path.split("\t")[0], genes=path.split("\t")[2:] if path.split("\t")[2:] else [], hierarchy=("Reactome", "Pathways"), organism="9606", link="") for path in content]
    return GeneSets(genesets)


def omimGeneSets():
    """
    Return gene sets from OMIM (Online Mendelian Inheritance in Man) diseses
    """
    genesets = [GeneSet(id=disease.id, name=disease.name, genes=obiOMIM.disease_genes(disease), hierarchy=("OMIM",), organism="9606",
                    link=("http://www.omim.org/entry/%s" % disease.id if disease.id else None)) \
                    for disease in obiOMIM.diseases()]
    return GeneSets(genesets)

def miRNAGeneSets(org):
    """
    Return gene sets from miRNA targets
    """
    from .. import obimiRNA
    org_code = obiKEGG.from_taxid(org)
    link_fmt = "http://www.mirbase.org/cgi-bin/mirna_entry.pl?acc=%s"
    mirnas = [(id, obimiRNA.get_info(id)) for id in obimiRNA.ids(org_code)]
    genesets = [GeneSet(id=mirna.matACC, name=mirna.matID, genes=mirna.targets.split(","), hierarchy=("miRNA", "Targets"),
                        organism=org, link=link_fmt % mirna.matID) for id, mirna in mirnas]
    return GeneSets(genesets)

def go_miRNASets(org, ontology=None, enrichment=True, pval=0.05, treshold=0.04):
    from .. import obimiRNA
    mirnas = obimiRNA.ids(int(org))
    if ontology is None:
        ontology = obiGO.Ontology()

    annotations = obiGO.Annotations(org, ontology=ontology)

    go_sets = obimiRNA.get_GO(mirnas, annotations, enrichment=enrichment, pval=pval, goSwitch=False)

    go_sets = obimiRNA.filter_GO(go_sets, annotations, treshold=treshold)

    link_fmt = "http://amigo.geneontology.org/cgi-bin/amigo/term-details.cgi?term=%s"
    gsets = [GeneSet(id=key, name=ontology[key].name, genes=value, hierarchy=("miRNA", "go_sets",),
                        organism=org, link=link_fmt % key) for key, value in go_sets.items()]
    gset = GeneSets(gsets)
    return gset

import re

linkre = re.compile("(.*?)\s?(?:\[(https?://[^[^\]]*)\])?$")

def loadGMT(contents, name):
    """
    Eech line consists of tab separated elements. First is
    the geneset name, next is it's description.

    For now the description is skipped.

    Example Gene Set (.gmt) file:
    anti-liver_sw   anti-liver_sw   BMPR1A  APOD    WSB1    BMI1    SLC2A1  ...
    B-cells_sw  B-cells_sw  E2F5    NCF1    PALM2-AKAP2 IRF4    SLC2A1  ...
    Bladder_sw  Bladder_sw  PLCD4   ANGPTL1 LOC286191   ST0N1   LOC283904   ...
    cerebellum_sw   cerebellum_sw   C19orf43    LOC653464   KI110802    ...
    Cervix_sw   Cervix_sw   LAMA4   GSTM5   SNX19   DKK1    NT5E    ...
    """

    def hline(s):
        tabs = [tab.strip() for tab in s.split("\t")]
        groups = linkre.match(tabs[1]).groups()
        return GeneSet(id=tabs[0], description=groups[0], link=groups[1],
                                   hierarchy=("Custom",name), genes=tabs[2:])

    def handleNELines(s, fn):
        """
        Run function on nonempty lines of a string.
        Return a list of results for each line.
        """
        lines = (l.strip() for l in s.splitlines())
        return [fn(l) for l in lines if l]

    return GeneSets(handleNELines(contents, hline))

def getGenesetsStats(genesets):
    num_sets = len(genesets)
    unique_genes = len(set([gene for geneset in genesets for gene in geneset.genes]))
    genes_per_geneset = sum([len(geneset.genes) for geneset in genesets])/num_sets
    return num_sets, unique_genes, genes_per_geneset
"""
We have multiple paths for gene set data:
buffer/bigfiles/gene_sets
and
buffer/gene_sets_local
both have available.txt
"""

def omakedirs(dir):
    try:
        os.makedirs(dir)
    except OSError:
        pass

def local_path():
    """ Returns local path for gene sets. Creates it if it does not exists
    yet. """
    pth = os.path.join(environ.buffer_dir, "gene_sets_local")
    omakedirs(pth)
    return pth

def build_index(dir):
    """ Returns gene set availability index for some folder. """
    pass

def filename(hierarchy, organism):
    """ Obtain a filename for given hierarchy and organism. """
    return "gs_" + "_._".join(hierarchy + \
        (organism if organism != None else "",)) + ".pck"

def filename_parse(fn):
    """ Returns a hierarchy and the organism from the filename."""
    fn = fn[3:-4]
    parts = fn.split("_._")
    hierarchy = tuple(parts[:-1])
    org = parts[-1] if parts[-1] != "" else None
    return hierarchy, org

def is_genesets_file(fn):
    return fn.startswith("gs_") and fn.endswith(".pck")

def list_local():
    """ Returns available gene sets from the local repository:
    a list of (hierarchy, organism, on_local) """
    pth = local_path()
    gs_files = filter(is_genesets_file, os.listdir(pth))
    return [ filename_parse(fn) + (True,) for fn in gs_files ]

def remove_local(gene_set):
    """ Removes a given gene set from the local repository. """
    pth = local_path()
    gs_files = filter(is_genesets_file, os.listdir(pth)) 
    for setfile in gs_files:
        if setfile.__contains__(gene_set):
            setBgone = os.path.join(pth, setfile)
            os.remove(setBgone) 

def modification_date(file):
    t = os.path.getmtime(file)
    return datetime.datetime.fromtimestamp(t)

def list_serverfiles_from_flist(flist):
    gs_files = filter(is_genesets_file, flist)
    localfiles = os.listdir(serverfiles.localpath(sfdomain))
    localfiles = set(filter(is_genesets_file, localfiles))
    return [ filename_parse(fn) + \
        ((True,) if fn in localfiles else (False,)) for fn in set(gs_files) | localfiles ]

def list_serverfiles_conn(serverfiles=None):
    """ Returns available gene sets from the server files
    repository: a list of (hierarchy, organism, on_local) """
    if serverfiles == None:
        serverfiles = serverfiles.ServerFiles()
    flist = serverfiles.listfiles(sfdomain)
    return list_serverfiles_from_flist(flist)

def list_serverfiles():
    fname = serverfiles.localpath_download(sfdomain, "index.pck")
    if six.PY3:
        flist = pickle.load(open(fname, 'rb'), encoding="latin1")
    else:
        flist = pickle.load(open(fname, 'rb'))
    return list_serverfiles_from_flist(flist)

def list_all(org=None, local=None):
    """
    Return gene sets available in the local and ServerFiles repositories. 
    It returns a list of tuples of (hierarchy, organism, available_locally)

    Results can be filtered with the following parameters.

    :param str org: Organism tax id.
    :param bool local: Available locally.
    """
    flist = list_local() + list_serverfiles()
    d = {}
    for h,o,l in flist:
        d[h,o] = min(l, d.get((h,o),True))
    return [ (h,o,l) for (h,o),l in d.items() \
            if (local == None or l == local) and \
               (org == None or o == str(org))
        ]

def update_server_list(serverfiles_upload, serverfiles_list=None):
    if serverfiles_list == None:
        serverfiles_list = serverfiles.ServerFiles()

    flist = map(lambda x: filename(*x[:2]), list_serverfiles_conn(serverfiles_list))

    tfname = pickle_temp(flist)
    try:
        fn = "index.pck"
        title = "Gene sets: index"
        tags = [ "gene sets", "index", "essential" ]
        serverfiles_upload.upload(sfdomain, fn, tfname, title, tags)
        serverfiles_upload.unprotect(sfdomain, fn)
    except Exception as e:
        raise e
    finally:
        os.remove(tfname)

def _register_local(genesets):
    """ Registers using the common hierarchy and organism. """
    pth = local_path()
    org = genesets.common_org()
    hierarchy = genesets.common_hierarchy()
    fn = filename(hierarchy, org)

    with open(os.path.join(pth, fn), "wb") as f:
        pickle.dump(genesets, f)

    return fn

def pickle_temp(obj):
    """ Pickle a file to a temporary file and returns its name """
    fd,tfname = tempfile.mkstemp()
    os.close(fd)
    f = open(tfname, 'wb')
    pickle.dump(obj, f)
    f.close()
    return tfname

def _register_serverfiles(genesets, serverFiles):
    """ Registers using the common hierarchy and organism. """
    org = genesets.common_org()
    hierarchy = genesets.common_hierarchy()
    fn = filename(hierarchy, org)

    #save to temporary file
    tfname = pickle_temp(genesets)
    try:
        if org != None:
            taxname = obiTaxonomy.name(org)
        title = "Gene sets: " + ", ".join(hierarchy) + \
            ((" (" + taxname + ")") if org != None else "")
        tags = list(hierarchy) + [ "gene sets" ] + ([ taxname ] if org != None else [])  + obiTaxonomy.shortname(org) +\
            ([ "essential" ] if org in obiTaxonomy.essential_taxids() else [] )
        serverFiles.upload(sfdomain, fn, tfname, title, tags)
        serverFiles.unprotect(sfdomain, fn)
    finally:
        os.remove(tfname)

    update_server_list(serverFiles)

def register(genesets, serverFiles=None):
    """ Registers given genesets locally.  The gene set is registered
    by the common hierarchy or organism (None if organisms are different).

    :param GeneSets genesets:
    :param serverFiles: If `serverFiles` is an authenticated ServerFiles connection,
        the input gene sets are uploaded to the repository.  
    """
    if serverFiles == None:
        _register_local(genesets)
    else:
        _register_serverfiles(genesets, serverFiles)

def build_hierarchy_dict(files):
    hierd = defaultdict(list)
    for ind,f in enumerate(files):
        hier, org = f
        for i in range(len(hier)+1):
            hierd[(hier[:i], org)].append(ind)
    return hierd

def load_local(hierarchy, organism):
    return load_fn(hierarchy, organism, list_local, 
        lambda h,o: os.path.join(local_path(), filename(h, o)))

def load_serverfiles(hierarchy, organism):
    return load_fn(hierarchy, organism, list_serverfiles, 
        lambda h,o: serverfiles.localpath_download(sfdomain, filename(h, o)))

def load_fn(hierarchy, organism, fnlist, fnget):
    files = list(map(lambda x: x[:2], fnlist()))
    hierd = build_hierarchy_dict(files)
    out = GeneSets()
    matches = hierd[(hierarchy, organism)]
    if not matches:
        exstr = "No gene sets for " + str(hierarchy) + \
                " (org " + str(organism) + ")"
        raise NoGenesetsException(exstr)
    for (h, o) in [ files[i] for i in hierd[(hierarchy, organism)]]:
        fname = fnget(h, o)
        if six.PY3:
            out.update(pickle.load(open(fname, 'rb'), encoding="latin1"))
        else:
            out.update(pickle.load(open(fname, 'rb')))
    return out

def load(hierarchy, organism):
    """ First try to load from the local registered folder. If the file
    is not available, load it from the server files. """
    if organism != None:
        try:
            int(organism) #already a taxid
        except:
            organismc = obiTaxonomy.to_taxid(strornone(organism))
            if len(organismc) == 1:
                organism = organismc.pop()
            else:
                exstr = "Could not interpret organism " + str(organism) + \
                      ". Possibilities: " + str(organismc) 
                raise NoGenesetsException(exstr)

    try:
        return load_local(hierarchy, strornone(organism))
    except NoGenesetsException:
        return load_serverfiles(hierarchy, strornone(organism))

def collections(*args):
    """
    Load gene sets from various sources: GMT file, GO, KEGG, and others. 
    Return an instance of :class:`GeneSets`. 
    
    Each arguments specifies a gene set and can be either:
    
    * a filename of a GMT file,
    * a tuple (hierarchy, organism) (for example ``(("KEGG",), "10090")``), or
    * an instance of :class:`GeneSets`
    """
    result = GeneSets()

    for collection in args:
        try:
            result.update(collection)
        except (ValueError, TypeError):
            if issequencens(collection): #have a hierarchy, organism specification
                new = load(*collection)
                result.update(new)
            else:
                if collection.lower()[-4:] == ".gmt": #format from webpage
                    result.update(loadGMT(open(collection,"rt").read(), collection))
                else:
                    raise Exception("collection() accepts files in .gmt format only.")

    return result

def issequencens(x):
    "Is x a sequence and not string ? We say it is if it has a __getitem__ method and it is not an instance of basestring."
    return hasattr(x, '__getitem__') and not isinstance(x, basestring)

class TException(Exception): pass

def upload_genesets(rsf):
    """
    Builds the default gene sets and
    """
    serverfiles.update_local_files()

    genesetsfn = [ goGeneSets, keggGeneSets, miRNAGeneSets]
    organisms = obiTaxonomy.common_taxids()
    for fn in genesetsfn:
        for org in organisms:
            try:
                print("Uploading ORG %s %s" % (org, fn))
                genesets = fn(org).split_by_hierarchy()
                for gs in genesets:
                    print("registering %s" % str(gs.common_hierarchy()))
                    register(gs, rsf) #server files
                    #register(gs)
                    print("successful %s" % str(gs.common_hierarchy()))
            except obiTaxonomy.UnknownSpeciesIdentifier:
                print("Organism ontology not available %s" % org)
            except GenesetRegException:
                print("Empty gene sets. %s" % org)


def only_option(a):
    if len(a) == 1:
        return list(a)[0]
    else:
        raise Exception()

class GenesetRegException(Exception): pass

class GeneSet(object):
    """ A single set of genes.
    """

    def __init__(self, genes=[], name=None, id=None, \
        description=None, link=None, organism=None, hierarchy=None, pair=None):
        """
        :param pair: Only for backward compatibility: convert a tuple (name, genes)
            into this object.
        """

        self.hierarchy = hierarchy     
        """ Hierarchy should be formated as a tuple, for example ``("GO", "biological_process")``"""

        self.genes = set(genes)
        """ A set of genes. Genes are strings. """

        self.name = name
        """ Gene set name. """

        self.id = id
        """ Short gene set ID. """

        self.description = description
        """ Gene set description. """

        self.link = link
        """ Link to further information about this gene set. """

        self.organism = organism
        """ Organism as a NCBI taxonomy ID. """

        if pair:
            self.id, self.genes = pair[0], set(pair[1])

    """
    the following functions are needed for sets of gene sets to be able
    to assess equality
    """

    def __hash__(self):
        return self.id.__hash__() + self.name.__hash__()

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def size(self):
        return len(self.genes)

    def cname(self, source=True, name=True):
        """ Return a gene set name with hieararchy. """
        oname = self.id
        if source and self.hierarchy:
            oname = "[ " + ", ".join(self.hierarchy) + " ] " + oname
        if name and self.name:
            oname = oname + " " + self.name
        return oname

    def to_odict(self, source=True, name=True):
        """
        For backward compatibility. Return a gene set as a tuple
        (id, list of genes).
        """
        return self.cname(source=source, name=name), self.genes

    def __repr__(self):
        return "GeneSet(" + ", ".join( [ 
            "id=" + str(self.id),
            "genes=" + str(self.genes),
            "name=" + str(self.name),
            "link=" + str(self.link),
            "hierarchy=" + str(self.hierarchy)
        ]) + ")"

class GeneSetIDException(Exception):
    pass

class GeneSets(set):
    """ A collection of gene sets: contains :class:`GeneSet` objects. 
    """
    
    def __init__(self, input=None):
        """
        If `input` is a dictionary, the gene sets are converted to the current format.
        """
        if input != None and len(input) > 0:
            self.update(input)

    def update(self, input):
        if input.__class__.__name__ == "GeneSets": #HACK: because location can appear different
            super(GeneSets, self).update(input)
        else:
            prepared_genesets = [] #parse them all before adding,
                                   #so that it fails on error
            if hasattr(input, "items"):
                for i, g in input.items():
                    prepared_genesets.append(GeneSet(pair=(i, g)))
            else:
                for i in input:
                    if isinstance(i, GeneSet):
                        prepared_genesets.append(i)
                    else:
                        i, g = i
                        prepared_genesets.append(GeneSet(pair=(i, g)))

            for g in prepared_genesets:
                self.add(g)

    def to_odict(self):
        """ Return gene sets in old dictionary format. """
        return dict(gs.to_odict() for gs in self)

    def set_hierarchy(self, hierarchy):
        """ Sets hierarchy for all gene sets. """
        for gs in self:
            gs.hierarchy = hierarchy

    def __repr__(self):
        return "GeneSets(" + set.__repr__(self) + ")"

    def common_org(self):
        """ Return a common organism. """
        if len(self) == 0:
            raise GenesetRegException("Empty gene sets.")

        organisms = set(a.organism for a in self)

        try:
            return only_option(organisms)
        except:
            raise GenesetRegException("multiple organisms: " + str(organisms))

    def hierarchies(self):
        """ Return all hierarchies. """
        if len(self) == 0:
            raise GenesetRegException("Empty gene sets.")
        return set(a.hierarchy for a in self)

    def common_hierarchy(self):
        """ Return a common hierarchy. """
        hierarchies = self.hierarchies()

        def common_hierarchy1(hierarchies):
            def hier(l): return set(map(lambda x: x[:currentl], hierarchies))
            currentl = max(map(len, hierarchies))
            while len(hier(currentl)) > 1:
                currentl -= 1
            return only_option(hier(currentl))

        return common_hierarchy1(hierarchies)

    def split_by_hierarchy(self):
        """ Split gene sets by hierarchies. Return a list of :class:`GeneSets` objects. """
        hd = dict((h,GeneSets()) for h in  self.hierarchies())
        for gs in self:
            hd[gs.hierarchy].add(gs)
        return hd.values()


if __name__ == "__main__":
    rsf = serverfiles.ServerFiles(username=sys.argv[1], password=sys.argv[2])
    upload_genesets(rsf)
    pass






import sys
import os
import re
import unicodedata
import operator
import math
from collections import defaultdict, namedtuple
from operator import itemgetter
from functools import reduce
from xml.sax.saxutils import escape

from contextlib import contextmanager

from PyQt4.QtGui import (
    QFrame, QHBoxLayout, QPlainTextEdit, QSyntaxHighlighter, QTextCharFormat,
    QTextCursor, QCompleter, QStandardItemModel, QSortFilterProxyModel,
    QStandardItem, QListView, QTreeView, QStyle, QStyledItemDelegate,
    QStyleOptionViewItemV4, QPalette, QColor, QApplication, QAction,
    QToolButton, QItemSelectionModel, QPlainTextDocumentLayout, QTextDocument,
    QRadioButton, QButtonGroup, QStyleOptionButton, QMenu, QDialog,
    QStackedWidget, QComboBox, QFileDialog
)

from PyQt4.QtCore import Qt, QEvent, QThread
from PyQt4.QtCore import pyqtSignal as Signal

import Orange

from Orange.widgets import widget, settings, gui
from Orange.widgets.utils.concurrent import (
    ThreadExecutor, Task, methodinvoke
)

from Orange.widgets.utils.itemmodels import VariableListModel
from Orange.widgets.utils.datacaching import data_hints

from orangecontrib.bio import taxonomy
from orangecontrib.bio import gene as geneinfo


class SaveSlot(QStandardItem):
    ModifiedRole = next(gui.OrangeUserRole)

    def __init__(self, name, savedata=None, modified=False):
        super(SaveSlot, self).__init__(name)

        self.savedata = savedata
        self.modified = modified
        self.document = None

    @property
    def name(self):
        return self.text()

    @property
    def modified(self):
        return bool(self.data(SaveSlot.ModifiedRole))

    @modified.setter
    def modified(self, state):
        self.setData(bool(state), SaveSlot.ModifiedRole)


class SavedSlotDelegate(QStyledItemDelegate):

    def paint(self, painter, option, index):
        option = QStyleOptionViewItemV4(option)
        self.initStyleOption(option, index)

        modified = bool(index.data(SaveSlot.ModifiedRole))
        if modified:
            option.palette.setColor(QPalette.Text, QColor(Qt.red))
            option.palette.setColor(QPalette.Highlight, QColor(Qt.darkRed))
            option.text = "*" + option.text

        if option.widget:
            widget = option.widget
            style = widget.style()
        else:
            widget = None
            style = QApplication.style()

        style.drawControl(QStyle.CE_ItemViewItem, option, painter, widget)


def radio_indicator_width(button):
    button.ensurePolished()
    style = button.style()
    option = QStyleOptionButton()
    button.initStyleOption(option)

    w = style.pixelMetric(QStyle.PM_ExclusiveIndicatorWidth, option, button)
    return w


class OWSelectGenes(widget.OWWidget):

    name = "Select Genes"
    description = "Select a specified subset of the input genes."
    icon = "../widgets/icons/SelectGenes.svg"
    inputs = [("Data", Orange.data.Table, "setData", widget.Default),
              ("Gene Subset", Orange.data.Table, "setGeneSubset")]
    outputs = [("Selected Data", Orange.data.Table)]

    SelectInput, SelectCustom = 0, 1
    CompletionRole = Qt.UserRole + 1

    settingsHandler = settings.DomainContextHandler()

    geneIndex = settings.ContextSetting(0)
    taxid = settings.ContextSetting(None)
    subsetGeneIndex = settings.ContextSetting(-1)

    savedSelections = settings.Setting([
        ("Example", ["MRE11A", "RAD51", "MLH1", "MSH2", "DMC1"])
    ])
    selectedSource = settings.Setting(SelectCustom)
    selectedSelectionIndex = settings.Setting(-1)
    completeOnSymbols = settings.Setting(True)

    autoCommit = settings.Setting(False)
    preserveOrder = settings.Setting(True)

    want_main_area = False

    def __init__(self, parent=None):
        super().__init__(parent)

        # Input variables that could contain names
        self.variables = VariableListModel(parent=self)
        # All gene names and their symbols
        self.geneNames = []
        # A list of (name, info) where name is from the input
        # (geneVar column) and info is the NCBIGeneInfo object if available
        # or None
        self.genes = []
        # Output changed flag
        self._changedFlag = False
        # Current gene names
        self.selection = []
        # Input data
        self.data = None
        self.subsetData = None
        # Input variables that could contain gene names from "Gene Subset"
        self.subsetVariables = VariableListModel(parent=self)
        # Selected subset variable index

        self.organisms = []
        self.taxidindex = {}
        self.geneinfo = (None, None)
        self._executor = ThreadExecutor()

        self._infotask = None

        box = gui.widgetBox(self.controlArea, "Gene Attribute")
        box.setToolTip("Column with gene names")
        self.attrsCombo = gui.comboBox(
            box, self, "geneIndex",
            callback=self._onGeneIndexChanged,
        )
        self.attrsCombo.setModel(self.variables)

        box = gui.widgetBox(self.controlArea, "Gene Selection")

        button1 = QRadioButton("Select genes from 'Gene Subset' input")
        button2 = QRadioButton("Select specified genes")

        box.layout().addWidget(button1)

        # Subset gene variable selection
        self.subsetbox = gui.widgetBox(box, None)
        offset = radio_indicator_width(button1)
        self.subsetbox.layout().setContentsMargins(offset, 0, 0, 0)
        self.subsetbox.setEnabled(
            self.selectedSource == OWSelectGenes.SelectInput)

        box1 = gui.widgetBox(self.subsetbox, "Gene Attribute")
        box1.setFlat(True)
        self.subsetVarCombo = gui.comboBox(
            box1, self, "subsetGeneIndex",
            callback=self._onSubsetGeneIndexChanged
        )
        self.subsetVarCombo.setModel(self.subsetVariables)
        self.subsetVarCombo.setToolTip(
            "Column with gene names in the 'Gene Subset' input"
        )
        gui.button(box1, self, "Copy genes to saved subsets",
                     callback=self.copyToSaved)

        gui.button(box1, self, "Append genes to current saved selection",
                     callback=self.appendToSelected)

        box.layout().addWidget(button2)

        self.selectedSourceButtons = group = QButtonGroup(box)
        group.addButton(button1, OWSelectGenes.SelectInput)
        group.addButton(button2, OWSelectGenes.SelectCustom)
        group.buttonClicked[int].connect(self._selectionSourceChanged)

        if self.selectedSource == OWSelectGenes.SelectInput:
            button1.setChecked(True)
        else:
            button2.setChecked(True)

        self.entrybox = gui.widgetBox(box, None)
        offset = radio_indicator_width(button2)
        self.entrybox.layout().setContentsMargins(offset, 0, 0, 0)

        self.entrybox.setEnabled(
            self.selectedSource == OWSelectGenes.SelectCustom)

        box = gui.widgetBox(self.entrybox, "Select Genes")
        box.setFlat(True)
        box.setToolTip("Enter gene names to select")
        box.layout().setSpacing(1)

        self.entryField = ListTextEdit(box)
        self.entryField.setTabChangesFocus(True)
        self.entryField.setDocument(self._createDocument())
        self.entryField.itemsChanged.connect(self._onItemsChanged)

        box.layout().addWidget(self.entryField)

        completer = ListCompleter(self)
        completer.setCompletionMode(QCompleter.PopupCompletion)
        completer.setCompletionRole(
            self.CompletionRole if self.completeOnSymbols else Qt.DisplayRole
        )
        completer.setCaseSensitivity(Qt.CaseInsensitive)
        completer.setMaxVisibleItems(10)

        popup = QTreeView()
        popup.setSelectionMode(QTreeView.ExtendedSelection)
        popup.setEditTriggers(QTreeView.NoEditTriggers)
        popup.setRootIsDecorated(False)
        popup.setAlternatingRowColors(True)
        popup.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        popup.setMaximumWidth(500)
        popup.header().setStretchLastSection(False)
        popup.header().hide()

        completer.setPopup(popup)
        completer.setModel(SetFilterProxyModel())

        self.entryField.setCompleter(completer)

        toolbar = QFrame()
        toolbar.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
        layout = QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(1)

        addaction = QAction("+", self)
        addmenu = QMenu()

        action = addmenu.addAction("Import names from gene sets...")
        action.triggered.connect(self.importGeneSet)

        action = addmenu.addAction("Import names from text file...")
        action.triggered.connect(self.importFromFile)

        addaction.setMenu(addmenu)

        def button(action, popupMode=None):
            b = QToolButton()
            b.setDefaultAction(action)
            if popupMode is not None:
                b.setPopupMode(QToolButton.InstantPopup)
            return b

        b = button(addaction, popupMode=QToolButton.InstantPopup)
        layout.addWidget(b)

        moreaction = QAction("More", self)

        moremenu = QMenu()
        self.completeOnSymbolsAction = QAction(
            "Complete on gene symbol names", self,
            toolTip="Use symbol names for auto completion.",
            checkable=True,
        )
        self.completeOnSymbolsAction.setChecked(self.completeOnSymbols)
        self.completeOnSymbolsAction.toggled[bool].connect(
            self._onToggleSymbolCompletion
        )

        moremenu.addAction(self.completeOnSymbolsAction)

        self.translateAction = QAction(
            "Translate all names to official symbol names", self,
            enabled=False
        )
        self.translateAction.triggered.connect(self._onTranslate)

        moremenu.addAction(self.translateAction)
        moreaction.setMenu(moremenu)

        b = button(moreaction, popupMode=QToolButton.InstantPopup)
        layout.addWidget(b)

        self.organismsCombo = ComboBox(focusPolicy=Qt.StrongFocus)
        self.organismsCombo.addItem("...")
        self.organismsCombo.model().item(0).setEnabled(False)
        self.organismsCombo.setMinimumWidth(200)
        self.organismsCombo.activated[int].connect(self._onOrganismActivated)

        layout.addSpacing(10)
        layout.addWidget(self.organismsCombo)
        self.pb = QProgressBar()
        self.pb.hide()
        self.pbstack = QStackedWidget()
        self.pbstack.addWidget(self.pb)

        layout.addSpacing(10)
        layout.addWidget(self.pbstack, 0, Qt.AlignRight | Qt.AlignVCenter)
        toolbar.setLayout(layout)

        box.layout().addWidget(toolbar)

        box = gui.widgetBox(self.entrybox, "Saved Selections")
        box.setFlat(True)
        box.setToolTip("Save/Select/Update saved gene selections")
        box.layout().setSpacing(1)

        self.selectionsModel = QStandardItemModel(self)
        self.selectionsView = QListView()
        self.selectionsView.setAlternatingRowColors(True)
        self.selectionsView.setModel(self.selectionsModel)
        self.selectionsView.setItemDelegate(SavedSlotDelegate(self))
        self.selectionsView.selectionModel().selectionChanged.connect(
            self._onSelectedSaveSlotChanged
        )

        box.layout().addWidget(self.selectionsView)

        self.actionSave = QAction(
            "Save", self,
            toolTip="Save/Update the current selection")

        self.actionAdd = QAction(
            "+", self,
            toolTip="Create a new saved selection")

        self.actionRemove = QAction(
            "\u2212", self,
            toolTip="Delete the current saved selection")

        toolbar = QFrame()
        layout = QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(1)

        b = button(self.actionAdd)
        layout.addWidget(b)

        b = button(self.actionRemove)
        layout.addWidget(b)

        b = button(self.actionSave)
        layout.addWidget(b)

        layout.addStretch(100)
        toolbar.setLayout(layout)

        box.layout().addWidget(toolbar)

        self.actionSave.triggered.connect(self.saveSelection)
        self.actionAdd.triggered.connect(self.addSelection)
        self.actionRemove.triggered.connect(self.removeSelection)

        box = gui.widgetBox(self.controlArea, "Output")
        gui.checkBox(box, self, "preserveOrder", "Preserve input order",
                     tooltip="Preserve the order of the input data "
                             "instances.",
                     callback=self.invalidateOutput)
        gui.auto_commit(box, self, "autoCommit", "Commit", box=None)
#         cb = gui.checkBox(box, self, "autoCommit", "Auto commit")
#         button = gui.button(box, self, "Commit", callback=self.commit)
#         gui.setStopper(self, button, cb, "_changedFlag", self.commit)

        # Gene set import dialog (initialized when required)
        self._genesetDialog = None

        # restore saved selections model.
        for name, names in self.savedSelections:
            item = SaveSlot(name, names)
            self.selectionsModel.appendRow([item])

        if self.selectedSelectionIndex != -1:
            self.selectionsView.selectionModel().select(
                self.selectionsModel.index(self.selectedSelectionIndex, 0),
                QItemSelectionModel.Select
            )

        self._updateActions()

        def list_available_organisms():
            return [(taxid, taxonomy.name(taxid))
                    for taxid in taxonomy.common_taxids()]

        self._inittask = Task(function=list_available_organisms)

        self._inittask.finished.connect(self._onInit)
        self._executor.submit(self._inittask)
        self.pb.show()
        self.pb.setRange(0, 0)

    def _onInit(self):
        self.organisms = self._inittask.result()
        self.organismsCombo.clear()
        self.organismsCombo.addItems([name for _, name in self.organisms])

        self.taxidindex = \
            {taxid: i for i, (taxid, _) in enumerate(self.organisms)}

        self.organismsCombo.setCurrentIndex(
            self.taxidindex.get(self.taxid, -1))

        self.pb.hide()

        if self.taxid in self.taxidindex:
            self._updateGeneInfo()

    def _updateGeneInfo(self):
        self.geneinfo = (self.taxid, None)
        self.translateAction.setEnabled(False)

        if self._infotask:
            # Try to cancel existing pending task
            self._infotask.future().cancel()
            self._infotask.resultReady.disconnect(self._onGeneInfoReady)
            self._infotask.exceptionReady.disconnect(self._onGeneInfoError)

        def ncbi_gene_info(taxid=self.taxid):
            try:
                return (taxid, geneinfo.NCBIGeneInfo(taxid))
            except BaseException:
                sys.excepthook(*sys.exc_info())
                raise

        task = Task(function=ncbi_gene_info)
        task.resultReady.connect(self._onGeneInfoReady)
        task.exceptionReady.connect(self._onGeneInfoError)
        self._infotask = task
        self._executor.submit(task)
        self.pb.show()

    def _onOrganismActivated(self, index):
        try:
            taxid, _ = self.organisms[index]
        except IndexError:
            pass
        else:
            if taxid != self.taxid:
                self.taxid = taxid
                self._updateGeneInfo()

    def setData(self, data):
        """
        Set the input data.
        """
        self.closeContext()
        self.warning(0)
        self.data = data
        if data is not None:
            attrs = gene_candidates(data)
            self.variables[:] = attrs
            self.attrsCombo.setCurrentIndex(0)
            if attrs:
                self.geneIndex = 0
            else:
                self.geneIndex = -1
                self.warning(0, "No suitable columns for gene names.")
        else:
            self.variables[:] = []
            self.geneIndex = -1

        self._changedFlag = True

        oldtaxid = self.taxid
        if data is not None:
            self.taxid = data_hints.get_hint(data, "taxid", None)
        else:
            self.taxid = None

        self.openContext(data)

        if self.taxid is None:
            self.geneinfo = (None, None)

        if oldtaxid != self.taxid:
            self.organismsCombo.setCurrentIndex(
                self.taxidindex.get(self.taxid, -1))

            if self.taxid in self.taxidindex:
                self._updateGeneInfo()

        self._updateCompletionModel()

        self.commit()

    def setGeneSubset(self, data):
        """
        Set the gene subset input.
        """
        # TODO: In Orange 2 we had two context handlers. This is no
        # longer possible in Orange3.
#         self.closeContext("subset")
        self.warning(1)
        self.subsetData = data
        if data is not None:
            variables = gene_candidates(data)
            self.subsetVariables[:] = variables
            self.subsetVarCombo.setCurrentIndex(0)
            if variables:
                self.subsetGeneIndex = 0
            else:
                self.subsetGeneIndex = -1
                self.warning(1, "No suitable column for subset gene names.")
        else:
            self.subsetVariables[:] = []
            self.subsetGeneIndex = -1

#         self.openContext("subset", data)

        if self.selectedSource == OWSelectGenes.SelectInput:
            self.commit()

    @property
    def geneVar(self):
        """
        Current gene attribute or None if none available.
        """
        index = self.attrsCombo.currentIndex()
        if self.data is not None and index >= 0:
            return self.variables[index]
        else:
            return None

    @property
    def subsetGeneVar(self):
        """
        Current subset gene attribute or None if not available.
        """
        index = self.subsetVarCombo.currentIndex()
        if self.subsetData is not None and index >= 0:
            return self.subsetVariables[index]
        else:
            return None

    def invalidateOutput(self):
        if self.autoCommit:
            self.commit()
        else:
            self._changedFlag = True

    def selectedGenes(self):
        """
        Return the names of the currently selected genes.
        """
        selection = []
        if self.selectedSource == OWSelectGenes.SelectInput:
            var = self.subsetGeneVar
            if var is not None:
                assert isinstance(var, Orange.data.StringVariable)
                values = [inst[var] for inst in self.subsetData]
                selection = [str(val) for val in values if not math.isnan(val)]
        else:
            selection = self.selection
        return selection

    def commit(self):
        """
        Send the selected data subset to the output.
        """
        selection = self.selectedGenes()

        if self.geneinfo[1] is not None:
            backmap = dict((info.symbol, name) for name, info in self.genes
                           if info is not None)
            selection = [backmap.get(name, name) for name in selection]
        if self.geneVar is not None:
            data = select_by_genes(self.data, self.geneVar,
                                   gene_list=selection,
                                   preserve_order=self.preserveOrder)
        else:
            data = None

        self.send("Selected Data", data)
        self._changedFlag = False

    def setSelectionSource(self, source):
        if self.selectedSource != source:
            self.selectedSource = source
            self.subsetbox.setEnabled(source == OWSelectGenes.SelectInput)
            self.entrybox.setEnabled(source == OWSelectGenes.SelectCustom)
            b = self.selectedSourceButtons.button(source)
            b.setChecked(True)

    def _selectionSourceChanged(self, source):
        if self.selectedSource != source:
            self.selectedSource = source
            self.subsetbox.setEnabled(source == OWSelectGenes.SelectInput)
            self.entrybox.setEnabled(source == OWSelectGenes.SelectCustom)
            self.invalidateOutput()

    def _updateCompletionModel(self):
        var = self.geneVar
        if var is not None:
            assert isinstance(var, Orange.data.StringVariable)
            names, _ = self.data.get_column_view(var)
            if names.dtype == object:
                names = [name for name in names if isinstance(name, str)]
            elif names.dtype.kind in ["U", "S"]:
                names = [str(name) for name in names]
            else:
                raise TypeError
        else:
            names = []

        infodict = {}

        if self.geneinfo[1] is not None:
            info = [(name, self.geneinfo[1].get_info(name, None))
                    for name in names]
            info = filter(itemgetter(1), info)
            infodict = dict(info)

        names = sorted(set(names))
        genes = list(zip(names, map(infodict.get, names)))

        symbols = [info.symbol for _, info in genes if info is not None]

        model = QStandardItemModel()

        def make_row(name, info):
            if info is not None:
                col1 = QStandardItem(name)
                col1.setData(info.symbol, OWSelectGenes.CompletionRole)
                return [col1,
                        QStandardItem(info.symbol),
                        QStandardItem(info.description)]
            else:
                col1 = QStandardItem(name)
                col1.setData(name, OWSelectGenes.CompletionRole)
                return [col1]

        for name, info in genes:
            model.appendRow(make_row(name, info))

        self.geneNames = sorted(set(names) | set(symbols))
        self.genes = genes

        self.entryField.completer().model().setSourceModel(model)
        self.entryField.document().highlighter.setNames(names + symbols)

        self._updatePopupSections()

    def _onGeneIndexChanged(self):
        self._updateCompletionModel()
        self.invalidateOutput()

    def _onSubsetGeneIndexChanged(self):
        if self.selectedSource == OWSelectGenes.SelectInput:
            self.invalidateOutput()

    def _onItemsChanged(self, names):
        selection = set(names).intersection(self.geneNames)
        curr_selection = set(self.selection).intersection(self.geneNames)

        self.selection = names

        if selection != curr_selection:
            self.invalidateOutput()
            to_complete = sorted(set(self.geneNames) - set(names))
            self.entryField.completer().model().setFilterFixedSet(to_complete)

        item = self._selectedSaveSlot()
        if item:
            item.modified = item.savedata != names

    def _onToggleSymbolCompletion(self, state):
        completer = self.entryField.completer()
        completer.setCompletionRole(
            self.CompletionRole if state else Qt.DisplayRole
        )
        self.completeOnSymbols = state
        self._updatePopupSections()

    def _onTranslate(self):
        if self.geneinfo[1] is not None:
            items = self.entryField.items()
            entries = map(self.geneinfo[1].get_info, items)
            items = [info.symbol if info is not None else item
                     for item, info in zip(items, entries)]
            self.entryField.selectAll()
            self.entryField.insertPlainText(" ".join(items))

    def _onGeneInfoReady(self, geneinfo):
        assert QThread.currentThread() is self.thread()
        # Check if the gene info is for the correct (current requested)
        # organism (we might receive a late response from a previous
        # request)
        if self.geneinfo[0] == geneinfo[0]:
            self.geneinfo = geneinfo
            self.translateAction.setEnabled(True)
            self.pb.hide()
            self._updateCompletionModel()

    def _onGeneInfoError(self, exc):
        self.error(0, str(exc))

    def _updatePopupSections(self):
        completer = self.entryField.completer()
        popup = completer.popup()
        assert isinstance(popup, QTreeView)
        header = popup.header()
        # The column in which the symbols should be
        symbol_col = 0 if self.completeOnSymbols else 1
        if symbol_col != header.sectionPosition(1):
            header.moveSection(0, 1)

    def _selectedSaveSlot(self):
        """
        Return the current selected saved selection slot.
        """
        indexes = self.selectionsView.selectedIndexes()
        if indexes:
            return self.selectionsModel.item(indexes[0].row())
        else:
            return None

    def saveSelection(self):
        """
        Save (update) the items in the current selected selection.
        """
        item = self._selectedSaveSlot()
        if item:
            item.savedata = self.entryField.items()
            item.modified = False

    def copyToSaved(self):
        """
        Copy the current 'Gene Subset' names to saved selections.
        """
        if self.subsetGeneVar and \
                self.selectedSource == OWSelectGenes.SelectInput:
            names = self.selectedGenes()
            item = SaveSlot("New selection")
            item.savedata = names
            self.selectionsModel.appendRow([item])
            self.setSelectionSource(OWSelectGenes.SelectCustom)
            self.selectionsView.setCurrentIndex(item.index())
            self.selectionsView.edit(item.index())

    def appendToSelected(self):
        """
        Append the current 'Gene Subset' names to 'Select Genes' entry field.
        """
        if self.subsetGeneVar and \
                self.selectedSource == OWSelectGenes.SelectInput:
            names = self.selectedGenes()
            text = " ".join(names)
            self.entryField.appendPlainText(text)
            self.setSelectionSource(OWSelectGenes.SelectCustom)
            self.entryField.setFocus()
            self.entryField.moveCursor(QTextCursor.End)

    def addSelection(self, name=None):
        """
        Add a new saved selection entry initialized by the current items.

        The new slot will be selected.

        """
        item = SaveSlot(name or "New selection")
        item.savedata = self.entryField.items()
        self.selectionsModel.appendRow([item])
        self.selectionsView.setCurrentIndex(item.index())

        if not name:
            self.selectionsView.edit(item.index())

    def removeSelection(self):
        """
        Remove the current selected save slot.
        """
        item = self._selectedSaveSlot()
        if item:
            self.selectionsModel.removeRow(item.row())

    def importGeneSet(self):
        if self._genesetDialog is None:
            self._genesetDialog = GeneSetDialog(
                self, windowTitle="Import Gene Set Names")

        dialog = self._genesetDialog

        if self.taxid is not None:
            dialog.setCurrentOrganism(self.taxid)

        result = dialog.exec_()
        if result == QDialog.Accepted:
            gsets = dialog.selectedGeneSets()
            genes = reduce(operator.ior, (gs.genes for gs in gsets), set())
            text = " ".join(genes)
            self.entryField.appendPlainText(text)
            self.entryField.setFocus()
            self.entryField.moveCursor(QTextCursor.End)

    def importFromFile(self):
        filename = QFileDialog.getOpenFileName(
            self, "Open File", os.path.expanduser("~/"))

        if filename:
            with open(filename, "rU") as f:
                text = f.read()
            self.entryField.appendPlainText(text)
            self.entryField.setFocus()
            self.entryField.moveCursor(QTextCursor.End)

    def _onSelectedSaveSlotChanged(self):
        item = self._selectedSaveSlot()
        if item:
            if not item.document:
                item.document = self._createDocument()
                if item.savedata:
                    item.document.setPlainText(" ".join(item.savedata))

            item.document.highlighter.setNames(self.geneNames)

            self.entryField.setDocument(item.document)

        self._updateActions()

    def _createDocument(self):
        """
        Create and new QTextDocument instance for editing gene names.
        """
        doc = QTextDocument(self)
        doc.setDocumentLayout(QPlainTextDocumentLayout(doc))
        doc.highlighter = NameHighlight(doc)
        return doc

    def _updateActions(self):
        """
        Update the Save/remove action enabled state.
        """
        selected = bool(self._selectedSaveSlot())
        self.actionRemove.setEnabled(selected)
        self.actionSave.setEnabled(selected)

    def getSettings(self, *args, **kwargs):
        # copy the saved selections model back to widget settings.
        selections = []
        for i in range(self.selectionsModel.rowCount()):
            item = self.selectionsModel.item(i)
            selections.append((item.name, item.savedata))
        self.savedSelections = selections

        item = self._selectedSaveSlot()
        if item is None:
            self.selectedSelectionIndex = -1
        else:
            self.selectedSelectionIndex = item.row()

        return OWWidget.getSettings(self, *args, **kwargs)

#     def sendReport(self):
#         report = []
#         if self.data is not None:
#             report.append("%i instances on input." % len(self.data))
#         else:
#             report.append("No data on input.")
# 
#         if self.geneVar is not None:
#             report.append("Gene names taken from %r attribute." %
#                           escape(self.geneVar.name))
# 
#         self.reportSection("Input")
#         self.startReportList()
#         for item in report:
#             self.addToReportList(item)
#         self.finishReportList()
#         report = []
#         selection = self.selectedGenes()
#         if self.selectedSource == OWSelectGenes.SelectInput:
#             self.reportRaw(
#                 "<p>Gene Selection (from 'Gene Subset' input): %s</p>" %
#                 escape(" ".join(selection))
#             )
#         else:
#             self.reportRaw(
#                 "<p>Gene Selection: %s</p>" %
#                 escape(" ".join(selection))
#             )
#         self.reportSettings(
#             "Settings",
#             [("Preserve order", self.preserveOrder)]
#         )

    def onDeleteWidget(self):
        self._inittask.future().cancel()

        if self._infotask:
            self._infotask.future().cancel()

        self._executor.shutdown(wait=True)
        super().onDeleteWidget()


def is_string(var):
    return isinstance(var, Orange.data.StringVariable)


def domain_variables(domain):
    """
    Return all feature descriptors from the domain.
    """
    return domain.variables + domain.metas


def gene_candidates(data):
    """
    Return features that could contain gene names.
    """
    return list(filter(is_string, domain_variables(data.domain)))


def select_by_genes(data, gene_feature, gene_list, preserve_order=True):
    if preserve_order:
        selection = set(gene_list)
        sel = [i for i, inst in enumerate(data)
               if str(inst[gene_feature]) in selection]
    else:
        by_genes = defaultdict(list)
        for i, inst in enumerate(data):
            by_genes[str(inst[gene_feature])].append(i)

        sel = []
        for name in gene_list:
            sel.extend(by_genes.get(name, []))

    return data.from_table(data.domain, data, sel)


_CompletionState = namedtuple(
    "_CompletionState",
    ["start",   # completion prefix start position
     "pos",     # cursor position
     "anchor"]  # anchor position (inline completion end)
)


class ListTextEdit(QPlainTextEdit):
    """
    A text editor specialized for editing a list of items.
    """
    #: Emitted when the list items change.
    itemsChanged = Signal(list)

    def __init__(self, parent=None, **kwargs):
        QPlainTextEdit.__init__(self, parent, **kwargs)

        self._items = None
        self._completer = None
        self._completionState = _CompletionState(-1, -1, -1)

        self.cursorPositionChanged.connect(self._cursorPositionChanged)
        self.textChanged.connect(self._textChanged)

    def setCompleter(self, completer):
        """
        Set a completer for list items.
        """
        if self._completer is not None:
            self._completer.setWidget(None)
            self._completer.activated.disconnect(self._insertCompletion)

        self._completer = completer

        if self._completer:
            self._completer.setWidget(self)
            self._completer.activated.connect(self._insertCompletion)

    def completer(self):
        """
        Return the completer.
        """
        return self._completer

    def setItems(self, items):
        text = " ".join(items)
        self.setPlainText(text)

    def items(self):
        if self._items is None:
            self._items = self._getItems()
        return self._items

    def keyPressEvent(self, event):
        # TODO: in Qt 4.8 QPlainTextEdit uses inputMethodEvent for
        # non-ascii input

        if self._completer.popup().isVisible():
            if event.key() in [Qt.Key_Enter, Qt.Key_Return, Qt.Key_Escape,
                               Qt.Key_Tab, Qt.Key_Backtab]:
                # These need to propagate to the completer.
                event.ignore()
                return

        QPlainTextEdit.keyPressEvent(self, event)

        if not len(event.text()) or not is_printable(event.text()[0]):
            return

        text = self.toPlainText()
        cursor = self.textCursor()
        pos = cursor.position()

        if pos == len(text) or not text[pos].strip():
            # cursor is at end of text or whitespace
            # find the beginning of the current word
            whitespace = " \t\n\r\f\v"
            start = max([text.rfind(c, 0, pos) for c in whitespace]) + 1

            prefix = text[start:pos]

            if prefix:
                if self._completer.completionPrefix() != prefix:
                    self._completer.setCompletionPrefix(text[start:pos])

                rect = self.cursorRect()
                popup = self._completer.popup()
                if popup.isVisible():
                    rect.setWidth(popup.width())
                else:
                    view_adjust_size_to_contents(popup)
                    rect.setWidth(popup.width())

                # Popup the completer list
                self._completer.complete(rect)

                # Inline completion of a common prefix
                inline = self._commonCompletionPrefix()
                inline = inline[len(prefix):]

                self._completionState = \
                    _CompletionState(start, pos, pos + len(inline))

                cursor.insertText(inline)
                cursor.setPosition(pos, QTextCursor.KeepAnchor)
                self.setTextCursor(cursor)

            elif self._completer.popup().isVisible():
                self._stopCompletion()

    def _cursorPositionChanged(self):
        cursor = self.textCursor()
        pos = cursor.position()
        start, _, _ = self._completionState

        if start == -1:
            # completion not in progress
            return

        if pos <= start:
            # cursor moved before the start of the prefix
            self._stopCompletion()
            return

        text = self.toPlainText()
        # Find the end of the word started by completion prefix
        word_end = len(text)
        for i in range(start, len(text)):
            if text[i] in " \t\n\r\f\v":
                word_end = i
                break

        if pos > word_end:
            # cursor moved past the word boundary
            self._stopCompletion()

        # TODO: Update the prefix when moving the cursor
        # inside the word

    def _insertCompletion(self, item):
        if isinstance(item, list):
            completion = " ".join(item)
        else:
            completion = item

        start, _, end = self._completionState

        self._stopCompletion()

        cursor = self.textCursor()
        # Replace the prefix+inline with the full completion
        # (correcting for the case-insensitive search).
        cursor.setPosition(min(end, self.document().characterCount()))
        cursor.setPosition(start, QTextCursor.KeepAnchor)

        cursor.insertText(completion + " ")

    def _commonCompletionPrefix(self):
        """
        Return the common prefix of items in the current completion model.
        """
        model = self._completer.completionModel()
        column = self._completer.completionColumn()
        role = self._completer.completionRole()
        items = [str(model.index(i, column).data(role))
                 for i in range(model.rowCount())]
        if not items:
            return ""

        first = min(items)
        last = max(items)
        for i, c in enumerate(first):
            if c != last[i]:
                return first[:i]

        return first

    def _stopCompletion(self):
        self._completionState = _CompletionState(-1, -1, -1)
        if self._completer.popup().isVisible():
            self._completer.popup().hide()

    def _textChanged(self):
        items = self._getItems()
        if self._items != items:
            self._items = items
            self.itemsChanged.emit(items)

    def _getItems(self):
        """
        Return the current items (a list of strings).

        .. note:: The inline completion text is not included.

        """
        text = self.toPlainText()
        if self._completionState[0] != -1:
            # Remove the inline completion text
            _, pos, end = self._completionState
            text = text[:pos] + text[end:]
        return [item for item in text.split() if item.strip()]


def view_adjust_column_sizes(view, max_width=None):
    """
    Adjust view's column sizes to to contents.
    """
    if max_width is None:
        max_width = sys.maxsize

    for col in range(view.model().columnCount()):
        width = min(view.sizeHintForColumn(col), max_width)
        view.setColumnWidth(col, width)


def view_adjust_size_to_contents(view):
    """
    Adjust the view to a reasonable size based in it's contents.
    """
    view_adjust_column_sizes(view)
    w = sum([view.columnWidth(col)
             for col in range(view.model().columnCount())])
    w += view.verticalScrollBar().sizeHint().width()

    h = view.sizeHintForRow(0) * 7
    h += view.horizontalScrollBar().sizeHint().height()
    view.resize(w, h)


class NameHighlight(QSyntaxHighlighter):
    def __init__(self, parent=None, **kwargs):
        super(NameHighlight, self).__init__(parent, **kwargs)

        self._names = set()

        self._format = QTextCharFormat()
        self._format.setForeground(Qt.blue)

        self._unrecognized_format = QTextCharFormat()
#         self._unrecognized_format.setFontStrikeOut(True)

    def setNames(self, names):
        self._names = set(names)
        self.rehighlight()

    def names(self):
        return set(self._names)

    def highlightBlock(self, text):
        pattern = re.compile(r"\S+")
        for match in pattern.finditer(text):
            name = text[match.start(): match.end()]
            match_len = match.end() - match.start()

            if not name.strip():
                continue

            if name in self._names:
                charfmt = self._format
            else:
                charfmt = self._unrecognized_format

            self.setFormat(match.start(), match_len, charfmt)


@contextmanager
def signals_blocked(obj):
    blocked = obj.signalsBlocked()
    obj.blockSignals(True)
    try:
        yield
    finally:
        obj.blockSignals(blocked)


class ListCompleter(QCompleter):
    """
    A completer supporting selection of multiple list items.
    """
    activated = Signal(list)

    def __init__(self, *args, **kwargs):
        QCompleter.__init__(self, *args, **kwargs)

        popup = QListView()
        popup.setEditTriggers(QListView.NoEditTriggers)
        popup.setSelectionMode(QListView.ExtendedSelection)

        self.setPopup(popup)

    def setPopup(self, popup):
        QCompleter.setPopup(self, popup)

        popup.viewport().installEventFilter(self)
        popup.doubleClicked.connect(self._complete)

    def eventFilter(self, receiver, event):
        if event.type() == QEvent.KeyPress and receiver is self.popup():
            if event.key() in [Qt.Key_Enter, Qt.Key_Return, Qt.Key_Tab]:
                self._complete()
                return True

        elif event.type() == QEvent.MouseButtonRelease and \
                receiver is self.popup().viewport():
            # Process the event without emitting 'clicked', ... signal to
            # override the default QCompleter behavior
            with signals_blocked(self.popup()):
                QApplication.sendEvent(self.popup(), event)
                return True

        return QCompleter.eventFilter(self, receiver, event)

    def _complete(self):
        column = self.completionColumn()
        role = self.completionRole()
        indexes = self.popup().selectionModel().selectedRows(column)

        items = [str(index.data(role)) for index in indexes]

        if self.popup().isVisible():
            self.popup().hide()

        if items:
            self.activated.emit(items)


class SetFilterProxyModel(QSortFilterProxyModel):
    def __init__(self, *args, **kwargs):
        super(SetFilterProxyModel, self).__init__(*args, **kwargs)
        self._filterFixedSet = None

    def setFilterFixedSet(self, items):
        if items is None:
            self._filterFixedSet = None
        else:
            self._filterFixedSet = set(items)
        self.invalidateFilter()

    def filterAcceptsRow(self, row, parent):
        if self._filterFixedSet is None:
            return True

        model = self.sourceModel()
        col = self.filterKeyColumn()
        var = model.data(model.index(row, col, parent),
                         self.filterRole())
        var = str(var)
        return var in self._filterFixedSet


class ComboBox(QComboBox):
    """
    A combo box ignoring mouse wheel events.
    """
    def wheelEvent(self, event):
        event.ignore()
        return

# All control character categories.
_control = set(["Cc", "Cf", "Cs", "Co", "Cn"])


def is_printable(unichar):
    """
    Return True if the unicode character `unichar` is a printable character.
    """
    return unicodedata.category(unichar) not in _control

import itertools

from PyQt4.QtGui import (
    QVBoxLayout, QLineEdit, QDialogButtonBox, QProgressBar, QSizePolicy
)

from PyQt4.QtCore import QSize
from orangecontrib.bio import geneset as genesets
from orangecontrib.bio.utils import serverfiles


class GeneSetView(QFrame):
    selectedOrganismChanged = Signal(str)
    selectionChanged = Signal()
    geneSetsLoaded = Signal()

    def __init__(self, *args, **kwargs):
        super(GeneSetView, self).__init__(*args, **kwargs)

        self._taxid = None

        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)

        self._stack = QStackedWidget()
        self._stack.setContentsMargins(0, 0, 0, 0)
        self._stack.setSizePolicy(QSizePolicy.MinimumExpanding,
                                  QSizePolicy.Fixed)
        self.orgcombo = ComboBox(minimumWidth=150, focusPolicy=Qt.StrongFocus)
        self.orgcombo.activated[int].connect(self._on_organismSelected)
        self._stack.addWidget(self.orgcombo)

        self.progressbar = QProgressBar()
        self._stack.addWidget(self.progressbar)

        layout.addWidget(self._stack)

        self.searchline = QLineEdit()
        self.searchline.setPlaceholderText("Filter...")
        completer = QCompleter()
        self.searchline.setCompleter(completer)
        layout.addWidget(self.searchline)

        self.gsview = QTreeView()
        self.gsview.setAlternatingRowColors(True)
        self.gsview.setRootIsDecorated(False)
        self.gsview.setSelectionMode(QTreeView.ExtendedSelection)
        self.gsview.setSortingEnabled(True)
        self.gsview.setUniformRowHeights(True)
        self.proxymodel = QSortFilterProxyModel(
            self, filterKeyColumn=1, sortCaseSensitivity=Qt.CaseInsensitive
        )

        self.gsview.setModel(self.proxymodel)
        self.gsview.selectionModel().selectionChanged.connect(
            self._on_selectionChanged)

        self.searchline.textChanged.connect(
            self.proxymodel.setFilterFixedString)

        layout.addWidget(self.gsview)
        self.setLayout(layout)

        self._executor = ThreadExecutor(self)
        self.initialize()

    def initialize(self):
        gs = genesets.list_all()
        # Filter out genesets with taxid == None
        # (What the furious felonious ferrous ferrets)
        gs = [(h, taxid, _2) for h, taxid, _2 in gs if taxid]

        self.gs_hierarchy = gs
        taxids = set(taxid for _, taxid, _ in gs)
        self.organisms = [(taxid, taxonomy.name(taxid)) for taxid in taxids]
        for taxid, name in self.organisms:
            self.orgcombo.addItem(name, taxid)

        self.orgcombo.setCurrentIndex(-1)

    def sizeHint(self):
        return QSize(500, 550)

    def setCurrentOrganism(self, taxid):
        taxids = [tid for tid, _ in self.organisms]
        if taxid is not None and taxid not in taxids:
            taxid = None

        if taxid != self._taxid:
            self._taxid = taxid
            if taxid is None:
                self.orgcombo.setCurrentIndex(-1)
            else:
                index = taxids.index(taxid)
                self.orgcombo.setCurrentIndex(index)
            self._updateGeneSetsModel()
            self.selectedOrganismChanged.emit(taxid)

    def currentOrganism(self):
        return self._taxid

    def selectedGeneSets(self):
        selmod = self.gsview.selectionModel()
        model = self.proxymodel.sourceModel()
        rows = [self.proxymodel.mapToSource(row)
                for row in selmod.selectedRows(1)]
        gsets = [model.data(row, Qt.UserRole) for row in rows]
        return gsets

    def _updateGeneSetsModel(self):
        taxid = self._taxid
        if taxid is None:
            self.proxymodel.setSourceModel(None)
        else:
            currentsets = [(hier, tid)
                           for hier, tid, _ in self.gs_hierarchy
                           if tid == taxid]

            gsmissing = [(hier, tid, local)
                         for hier, tid, local in self.gs_hierarchy
                         if tid == taxid and not local]

            self._stack.setCurrentWidget(self.progressbar)

            if gsmissing:
                self.progressbar.setRange(0, 100)
                progress_info = methodinvoke(
                    self.progressbar, "setValue", (int,))
            else:
                self.progressbar.setRange(0, 0)
                progress_info = None

            def load():
                gs_ensure_downloaded(
                    gsmissing,
                    progress_info=progress_info)

                return [((hier, tid), genesets.load(hier, tid))
                        for hier, tid in currentsets]

            self._task = Task(function=load)
            self._task.finished.connect(self._on_loadFinished)
            self._executor.submit(self._task)

    def _on_loadFinished(self):
        assert QThread.currentThread() is self.thread()
        self._stack.setCurrentWidget(self.orgcombo)

        try:
            sets = self._task.result()
        except Exception:
            # Should do something better here.
            sys.excepthook(*sys.exc_info())
            sets = []

        model = sets_to_model(sets)
        self.proxymodel.setSourceModel(model)
        self.gsview.resizeColumnToContents(0)
        self.geneSetsLoaded.emit()

    def _on_organismSelected(self, index):
        if index != -1:
            item = self.orgcombo.model().item(index)
            taxid = str(item.data(Qt.UserRole))
            self.setCurrentOrganism(taxid)

    def _on_selectionChanged(self, *args):
        self.selectionChanged.emit()


def sets_to_model(gsets):
    model = QStandardItemModel()
    model.setHorizontalHeaderLabels(["Category", "Name", "#"])

    for (hier, tid), sets in gsets:
        for gset in sets:
            ngenes = len(gset.genes)
            names = [escape(name) for name in list(gset.genes)[:30]]
            names = ", ".join(names)
            tooltip = "<p>{0}</p>{1}".format(escape(gset.name), names)
            if ngenes > 30:
                tooltip += ", ... ({0} names not shown)".format(ngenes - 30)

            category = QStandardItem(" ".join(hier))
            category.setData((hier, tid), Qt.UserRole)
            category.setEditable(False)
            category.setToolTip(tooltip)
            name = QStandardItem(gset.name)
            name.setData(gset, Qt.UserRole)
            name.setEditable(False)
            name.setToolTip(tooltip)
            count = QStandardItem()
            count.setData(ngenes, Qt.DisplayRole)
            count.setToolTip(tooltip)
            model.appendRow([category, name, count])

    return model


def gs_ensure_downloaded(gslist, progress_info=None):
    hierlist = [(hier, taxid) for hier, taxid, local in gslist
                if not local]

    files = [(genesets.sfdomain, genesets.filename(hier, taxid))
             for hier, taxid in hierlist]

    download_list(files, progress_info)


def download_list(files_list, progress_callback=None):
    nfiles = len(files_list)
    count = 100 * nfiles
    counter = itertools.count()

    def advance():
        progress_callback(100.0 * next(counter) / count)

    for domain, filename in files_list:
        serverfiles.download(domain, filename,
                             callback=advance if progress_callback else None)


class GeneSetDialog(QDialog):
    selectionChanged = Signal()

    def __init__(self, *args, **kwargs):
        super(GeneSetDialog, self).__init__(*args, **kwargs)
        layout = QVBoxLayout()
        layout.setContentsMargins(4, 4, 4, 4)
        self.gsview = GeneSetView()
        self.gsview.selectionChanged.connect(self.selectionChanged)

        layout.addWidget(self.gsview)

        buttonbox = QDialogButtonBox(
            QDialogButtonBox.Ok | QDialogButtonBox.Cancel,
            Qt.Horizontal
        )
        buttonbox.accepted.connect(self.accept)
        buttonbox.rejected.connect(self.reject)

        layout.addWidget(buttonbox)

        self.setLayout(layout)

    def selectedGeneSets(self):
        return self.gsview.selectedGeneSets()

    def setCurrentOrganism(self, taxid):
        self.gsview.setCurrentOrganism(taxid)

    def currentOrganism(self):
        return self.gsview.currentOrganism()


def test1():
    app = QApplication([])
    dlg = GeneSetDialog()
    dlg.show()
    dlg.raise_()
    dlg.exec_()
    del dlg
    app.processEvents()


def test():
    app = QApplication([])
    w = OWSelectGenes()
    data = Orange.data.Table("brown-selected")
    w.setData(data)
    w.setGeneSubset(Orange.data.Table(data[:10]))
    w.show()
    app.exec_()
    w.saveSettings()
    w.deleteLater()
    del w
    app.processEvents()

if __name__ == "__main__":
    test()




import sys
import os
import time

from collections import defaultdict

from PyQt4 import QtCore, QtGui
from PyQt4.QtGui import QTreeWidget, QTreeWidgetItem

import Orange.data

from Orange.widgets import widget, gui, settings
from Orange.widgets.utils.datacaching import data_hints

from ..utils import environ
from .. import dicty


# set buffer file
bufferpath = os.path.join(environ.buffer_dir, "dicty")
try:
    os.makedirs(bufferpath)
except OSError:
    pass

bufferfile = os.path.join(bufferpath, "database.sq3")


class OWDicty(widget.OWWidget):
    name = "dictyExpress"
    description = "Access to data in dictyExpress database."
    icon = "../widgets/icons/DictyExpress.svg"
    priority = 40

    inputs = []
    outputs = [("Data", Orange.data.Table)]

    serverToken = settings.Setting("")
    platform = settings.Setting(None)
    selectedExperiemtns = settings.Setting([])
    server = settings.Setting(dicty.defaddress)
    excludeconstant = settings.Setting(False)

    def __init__(self, parent=None):
        super().__init__(parent)

        self.buffer = dicty.CacheSQLite(bufferfile)

        self.dbc = None
        self.chipsl = []
        self.items = []
        self.searchString = ""

        box = gui.widgetBox(self.controlArea, "Cache")
        gui.button(box, self, "Clear cache", callback=self.clear_buffer)

        gui.checkBox(self.controlArea, self, "excludeconstant",
                     "Exclude labels with constant values")

        gui.button(self.controlArea, self, "&Commit", callback=self.Commit)
        box = gui.widgetBox(self.controlArea, "Server")

        gui.lineEdit(box, self, "serverToken", "Token", callback=self.Connect)
        gui.rubber(self.controlArea)

        gui.lineEdit(self.mainArea, self, "searchString", "Search",
                     callbackOnType=True, callback=self.SearchUpdate)

        self.experimentsWidget = QTreeWidget(
            selectionMode=QTreeWidget.ExtendedSelection,
            rootIsDecorated=True,
            sortingEnabled=True)

        self.experimentsWidget.setHeaderLabels(
            ["Strain", "Treatment", "Growth condition",
             "Platform", "N", "Chips"])

        self.mainArea.layout().addWidget(self.experimentsWidget)

        QtCore.QTimer.singleShot(0, self.UpdateExperiments)

    def sizeHint(self):
        return QtCore.QSize(800, 600)

    def __updateSelectionList(self, oldList, oldSelection, newList):
        oldList = [oldList[i] for i in oldSelection]
        return [i for i, new in enumerate(newList) if new in oldList]

    def Connect(self):
        address = self.server
        if not address.endswith("?"):
            address = address + "?"

        if self.serverToken:
            address += "token=" + self.serverToken + "&"
        try:
            self.dbc = dicty.DatabaseConnection(address, cache=self.buffer)
        except Exception as ex:
            sys.excepthook(*sys.exc_info())
            self.error(0, "Error connecting to server" + str(ex))
            return
        self.error(0)

    def clear_buffer(self):
        self.buffer.clear()
        self.UpdateExperiments()

    def UpdateExperiments(self):
        self.chipsl = []
        self.experimentsWidget.clear()
        self.items = []

        if not self.dbc:
            self.Connect()

        annotations = self.dbc.annotations("norms", None)

        elements = []

        for chip, annot in annotations:
            d = dict(annot)
            elements.append(((d.get("treatment", ""),
                              d.get("growthCond", ""),
                              d.get("platform", ""),
                              d.get("sample", "")),
                             chip))

        def different_chips(li):
            # Returns a map, where keys are different elements in li and
            # values their chip ids
            dc = defaultdict(list)
            for a, chip in li:
                dc[a].append(chip)
            return dc

        typeswchips = different_chips(elements)  # types with counts

        for (treatment, cond, platform, strain), cchips in typeswchips.items():
            self.chipsl.append(cchips)
            num = len(cchips)
            experiment = [strain, treatment, cond, platform,
                          str(num), ','.join(cchips)]
            self.items.append(QTreeWidgetItem(
                self.experimentsWidget, experiment))

        for i in range(5):
            self.experimentsWidget.resizeColumnToContents(i)

        self.progressBarFinished()

    def SearchUpdate(self):

        def matches(item, text):
            return any(text.upper() in str(item.text(i)).upper()
                       for i in range(item.columnCount()))

        for item in self.items:
            item.setHidden(any(not matches(item, s)
                               for s in self.searchString.split()))

    def Commit(self):
        if not self.dbc:
            self.Connect()

        start = time.time()

        pb = gui.ProgressBar(self, iterations=1000)

        ids = []
        for item in self.experimentsWidget.selectedItems():
            ids += str(item.text(5)).split(",")

        table = self.dbc.get_single_data(
            ids=ids, exclude_constant_labels=self.excludeconstant,
            callback=pb.advance)

        end = int(time.time() - start)

        pb.finish()

        data_hints.set_hint(table, "taxid", "352472", 10.0)
        data_hints.set_hint(table, "genesinrows", False, 10.0)

        self.send("Data", table)


def test_main(argv=sys.argv):
    app = QtGui.QApplication(argv)
    w = OWDicty()
    w.show()
    r = app.exec_()
    w.saveSettings()
    return r

if __name__ == "__main__":
    sys.exit(test_main())
